﻿using BPA.SAAS.Manage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.Manage.Application.DataBase.Dtos;
using BPA.SAAS.Manage.Application.DataBase.Dtos.Batching;
using BPA.SAAS.Manage.Application.DataBase.Dtos.Bom;
using BPA.SAAS.Manage.Application.DataBase.Dtos.Goods;
using BPA.SAAS.Manage.Application.DataBase.Dtos.GoodsAttribute;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Application.Device.Dtos.Device;
using BPA.SAAS.Manage.Application.System.Dtos;
using BPA.SAAS.Manage.Application.System.Interface;
using BPA.SAAS.Manage.Application.System.Services;
using BPA.SAAS.Manage.Comm.Const;
using BPA.SAAS.Manage.Comm.Enum;
using BPA.SAAS.Manage.Comm.Util;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using BPA.SAAS.Manage.Core.Device;
using BPA.SAAS.Manage.Core.Product;
using Furion.ClayObject.Extensions;
using Microsoft.AspNetCore.Hosting;
using Newtonsoft.Json;
using Npoi.Mapper;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.SS.Util;
using NPOI.XSSF.UserModel;
using Org.BouncyCastle.Asn1.Cms;
using System;
using System.Collections;
using System.ComponentModel.DataAnnotations.Schema;
using System.Data;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO.Compression;
using System.Linq;
using System.Net;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;

namespace BPA.SAAS.Manage.Application.DataBase.Services
{
    public class GoodsService : IGoodsService, ITransient
    {
        private readonly ISqlSugarClient _db;
        IGoodsAttributeService _goodsAttributeService;
        ISystemConfigService _SystemConfigService;
        IAliyunOssService _aliyunOssService;
        public GoodsService(ISqlSugarClient db, IGoodsAttributeService goodsAttributeService, ISystemConfigService SystemConfigService, IAliyunOssService aliyunOssService, IWebHostEnvironment webHostEnvironment)
        {
            _db = db;
            _goodsAttributeService = goodsAttributeService;
            _SystemConfigService = SystemConfigService;
            _aliyunOssService = aliyunOssService;
        }
        /// <summary>
        /// 分页查询
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetGoodsPage(GoodsQueryDto dto)
        {
            List<IConditionalModel> conModels = new List<IConditionalModel>();
            #region 条件查询
            if (!string.IsNullOrEmpty(dto.Name))
            {
                conModels.Add(new ConditionalModel() { FieldName = "a.Name", ConditionalType = ConditionalType.Like, FieldValue = dto.Name });
            }

            if (!string.IsNullOrEmpty(dto.GoodsTypeName))
            {
                conModels.Add(new ConditionalModel() { FieldName = "a.GoodsTypeId", ConditionalType = ConditionalType.Equal, FieldValue = dto.GoodsTypeName });
            }

            if (!string.IsNullOrEmpty(dto.Status))
            {
                conModels.Add(new ConditionalModel() { FieldName = "a.Status", ConditionalType = ConditionalType.Like, FieldValue = dto.Status });
            }
            if (!string.IsNullOrEmpty(dto.Descritption))
            {
                conModels.Add(new ConditionalModel() { FieldName = "a.Descritption", ConditionalType = ConditionalType.Like, FieldValue = dto.Descritption });
            }
            #endregion

            RefAsync<int> total = 0;
            var res = await _db.Queryable<BPA_GoodsInfo, BPA_GoodsType>((a, b) => new JoinQueryInfos(
                      JoinType.Left, a.GoodsTypeId == b.Id
                  ))
                .WhereIF(!string.IsNullOrWhiteSpace(dto.Code), (a, b) => a.Code.Contains(dto.Code))
                .Where(conModels)
                .OrderBy(a => a.CreateAt, OrderByType.Desc)
                .Select((a, b) => new
                {
                    Id = a.Id,
                    Code = a.Code,
                    Name = a.Name,
                    Price = a.Price,
                    ImgUrl = a.ImgUrl,
                    Status = a.Status,
                    GoodsTypeId = a.GoodsTypeId,
                    GoodsTypeName = b.Name,
                    Descritption = a.Descritption,
                    // CreateAt = a.CreateAt,
                    GoodsUintId = a.GoodsUintId,
                    ForeignKeyRe = a.ForeignKeyRe,
                    Design = a.Design,
                    IsWeigh = a.IsWeigh,
                    DefaultMate = a.DefaultMate,
                    IsAttrubute = a.IsAttrubute,
                    Goodstechnology = SqlFunc.Subqueryable<BPA_GoodsTechnologyAction>().Where(p => p.GoodsId == a.Id).WhereIF(!string.IsNullOrWhiteSpace(dto.DeviceId), p => p.DeviceId == dto.DeviceId).ToList(),
                }).ToListAsync();
            if (!string.IsNullOrWhiteSpace(dto.DeviceId))
            {
                var goodsId = await _db.Queryable<BPA_GoodsTechnologyAction>().Where(t => t.DeviceId == dto.DeviceId).Select(q => q.GoodsId).Distinct().ToListAsync();
                res = res.Where(t => goodsId.Contains(t.Id)).ToList();
            }
            PageUtil util = new PageUtil()
            {
                Total = total,
                Data = res
            };
            return util;
        }


        /// <summary>
        /// 获取商品信息（根据设备id）
        /// </summary>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<object> GetGoodsListByIds(string autoKey)
        {
            var device = await _db.Queryable<BPA_DeviceInfo>().FirstAsync(t => t.AutoKey.ToString() == autoKey);
            var goodsIds = await _db.Queryable<BPA_GoodsTechnologyAction>().Where(t => t.DeviceId == device.Id).Select(a => a.GoodsId).Distinct().ToListAsync();
            var result = new List<GoodsTypeResultDto>();
            //商品信息
            var goodsList = await _db.Queryable<BPA_GoodsInfo>().Where(x => goodsIds.Contains(x.Id)).ToListAsync();
            var typeIds = goodsList.Select(x => x.GoodsTypeId).ToList();
            //工艺分类
            var typeList = await _db.Queryable<BPA_GoodsType>().Where(x => typeIds.Contains(x.Id)).ToListAsync();
            //工艺分类属性
            var goodsAttribute = await _db.Queryable<BPA_GoodsAttribute>()
                .Where(x => typeIds.Contains(x.GoodsTypeId)).ToListAsync();
            var goodsAttributeIds = await _db.Queryable<BPA_GoodsAttribute>().Select(x => x.Id).ToListAsync();
            var goodsAttributeValue = await _db.Queryable<BPA_GoodsAttributeValue>()
                .Where(x => goodsAttributeIds.Contains(x.GoodsAttributeId))
                .Select(x => new GoodsAttributeValueResultDto()
                {
                    GoodsAttributeValueId = x.Id,
                    AttributeValue = x.AttributeValue,
                    GoodsAttributeId = x.GoodsAttributeId,
                    Sort = x.Sort
                })
                .ToListAsync();
            var goodsInfos = await _db.Queryable<BPA_GoodsInfo>().Where(x => typeIds.Contains(x.GoodsTypeId) && goodsIds.Contains(x.Id)).ToListAsync();
            var goodsTechnologyAction = await _db.Queryable<BPA_GoodsTechnologyAction>().ToListAsync();
            foreach (var type in typeList)
            {

                var value1 = goodsAttribute.Where(x => x.GoodsTypeId == type.Id)
                    .Select(x => new GoodsAttributeResultDto()
                    {
                        GoodsAttributeId = x.Id,
                        GoodsAttributeName = x.AttributeName,
                        Sort = x.Sort
                    }).OrderBy(t => t.Sort).ToList();
                foreach (var item in value1)
                {
                    var value2 = goodsAttributeValue.Where(x => x.GoodsAttributeId == item.GoodsAttributeId)
                        .Select(x => new GoodsAttributeValueResultDto()
                        {
                            AttributeValue = x.AttributeValue,
                            GoodsAttributeId = x.GoodsAttributeId,
                            GoodsAttributeValueId = x.GoodsAttributeValueId,
                            Sort = x.Sort
                        }).OrderBy(t => t.Sort).ToList();
                    item.GoodsAttributeValueList = value2;
                }
                var value3 = goodsInfos.Where(x => x.GoodsTypeId == type.Id)
                    .Select(x => new GoodsInfoResultDto()
                    {
                        GoodsId = x.Id,
                        GoodsName = x.Name,
                        ImgUrl = x.ImgUrl,
                    })
                    .ToList();
                foreach (var item in value3)
                {
                    var value4 = goodsTechnologyAction
                        .Where(x => x.GoodsId == item.GoodsId && x.DeviceId == device.Id)
                        .Select(x => new GoodsTechnologyActionResultDto()
                        {
                            GroupId = x.GroupId,
                            ActionJson = x.ActionJson,
                            ChnologyId = x.ChnologyId,
                            DeviceId = x.DeviceId,
                            GoodsAttributeId = x.GoodsAttributeId,
                            GoodsId = x.GoodsId,
                            IsBatch = x.IsBatch,
                            Sort = x.Sort,
                            StepName = x.StepName,
                            WarehousrTemplateId = x.WarehousrTemplateId,
                        }).ToList();
                    item.GoodsTechnologyActionList = value4;
                }
                result.Add(new GoodsTypeResultDto()
                {
                    GoodsTypeId = type.Id,
                    GoodsTypeName = type.Name,
                    GoodsAttributeList = value1,
                    GoodsInfoList = value3

                });
            }
            return result;
        }

        /// <summary>
        /// 获取商品分类信息（根据设备id）
        /// </summary>
        /// <param name="autoKey"></param>
        /// <returns></returns>
        public async Task<object> GetGoodsClassifyByIds(string autoKey)
        {
            var device = await _db.Queryable<BPA_DeviceInfo>().FirstAsync(t => t.AutoKey.ToString() == autoKey);
            var goodsIds = await _db.Queryable<BPA_GoodsTechnologyAction>().Where(t => t.DeviceId == device.Id).Select(a => a.GoodsId).Distinct().ToListAsync();
            var goodsList = await _db.Queryable<BPA_GoodsInfo>().ToListAsync();
            var classifyRelationList = await _db.Queryable<BPA_GoodsClassifyRelation>().ToListAsync();
            var classifyList = await _db.Queryable<BPA_GoodsClassify>().ToListAsync();
            var classifyDataList = new List<GoodsClassifyResultDto>();
            foreach (var classify in classifyList)
            {

                classifyDataList.Add(new GoodsClassifyResultDto
                {
                    GoodsClassifyId = classify.Id,
                    GoodsClassifyName = classify.Name,
                    GoodsList = classifyRelationList.Where(t => t.ClassifyId == classify.Id)
                    .Select(a => new GoodsDataResultDto { 
                        GoodsId = a.GoodsId, 
                        GoodsName = goodsList.FirstOrDefault(t => t.Id == a.GoodsId)?.Name, 
                        ImgUrl = goodsList.FirstOrDefault(t => t.Id == a.GoodsId)?.ImgUrl }).ToList()
                });
            }
            foreach (var item in classifyDataList)
            {
                item.GoodsList = item.GoodsList.Where(t => goodsIds.Contains(t.GoodsId)).ToList();
            }
            classifyDataList.RemoveAll(t => !t.GoodsList.Any());
            return classifyDataList;
        }

        /// <summary>
        /// 查询所有商品
        /// </summary>
        /// <returns></returns>
        public async Task<List<BPA_GoodsInfo>> GetGoodsList()
        {
            var res = await _db.Queryable<BPA_GoodsInfo>().OrderByDescending(t => t.CreateAt).ToListAsync();
            return res;
        }
        /// <summary>
        /// 添加商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> AddGoods(GoodsDto dto)
        {
            var old = await _db.Queryable<BPA_GoodsInfo>().FirstAsync(t => t.Name == dto.Name && t.GoodsTypeId == dto.GoodsTypeId);
            if (old != null)
                throw Oops.Oh("已存在相同商品，添加失败！");
            if (string.IsNullOrWhiteSpace(dto.Id))
            {
                var resEntity = new BPA_GoodsInfo();
                resEntity.Id = Guid.NewGuid().ToString();
                resEntity.GoodsTypeId = dto.GoodsTypeId;
                resEntity.Name = dto.Name;
                resEntity.Descritption = dto.Descritption;
                resEntity.ImgUrl = dto.ImgUrl;
                resEntity.Price = dto.Price;
                resEntity.IsWeigh = dto.IsWeigh;
                resEntity.GoodsUintId = dto.GoodsUintId;
                resEntity.IsAttrubute = dto.IsAttrubute;
                resEntity.Code = GetNumber2(8);
                var data = _db.Queryable<BPA_GoodsInfo>().Max(x => x.AutoKey);
                //if (data == 0) data = 1000;
                //else data = data + 1;
                // resEntity.AutoKey = data;
                var res = await _db.Insertable(resEntity).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                return res > 0;
            }
            else
            {
                return await UpdateGoods(dto);
            }

        }

        /// <summary>
        /// 更新商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateGoods(GoodsDto dto)
        {
            // 查询数据库中是否存在未删除的工艺分类
            var resEntity = _db.Queryable<BPA_GoodsInfo>().First(it => it.Id == dto.Id);
            if (null == resEntity)
            {
                throw Oops.Oh("商品不存在");
            }
            resEntity.GoodsTypeId = dto.GoodsTypeId;
            resEntity.Name = dto.Name;
            resEntity.Descritption = dto.Descritption;
            resEntity.ImgUrl = dto.ImgUrl;
            resEntity.Price = dto.Price;
            resEntity.IsWeigh = dto.IsWeigh;
            resEntity.GoodsUintId = dto.GoodsUintId;
            if (!string.IsNullOrWhiteSpace(dto.Status))
            {
                resEntity.Status = (CommonStatus)Enum.ToObject(typeof(CommonStatus), int.Parse(dto.Status));
            }

            var res = await _db.Updateable(resEntity).ExecuteCommandAsync();
            return res > 0;
        }
        /// <summary>
        /// 删除商品
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<bool> DeleteGoods(string id)
        {
            var res = 0;
            try
            {
                _db.Ado.BeginTran();
                var ids = id.Split(",");
                foreach (var idItem in ids)
                {
                    var good = _db.Queryable<BPA_GoodsInfo>().Where(x => x.Id == idItem).First();
                    if (good == null)
                        throw Oops.Oh("商品不存在");
                    var goodsbom = _db.Queryable<BPA_GoodsBom>().Where(x => ids.Contains(x.Goods_Id)).ToList();
                    var goodsTechnology = _db.Queryable<BPA_GoodsTechnologyAction>().Where(x => ids.Contains(x.GoodsId)).ToList();
                    await _db.Deleteable(goodsbom).ExecuteCommandAsync();
                    await _db.Deleteable(goodsTechnology).ExecuteCommandAsync();
                    res += await _db.Deleteable(good).ExecuteCommandAsync();
                }
                _db.Ado.CommitTran();
                return res > 0;
            }
            catch (Exception)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh("删除失败");
            }
        }
        /// <summary>
        /// 查询商品单位
        /// </summary>
        /// <returns></returns>
        public async Task<List<dynamic>> GetGoodsUintList()
        {
            var res = await _db.Queryable<BPA_GoodsUint>().Select<dynamic>(x => new

            {
                id = x.Id,
                name = x.Name
            }).ToListAsync();
            return res;
        }
        public async Task<List<dynamic>> GetGoodsUintList_alm()
        {
            var groupId = App.HttpContext.Request.Headers["groupId"].ToString();
            if (string.IsNullOrWhiteSpace(groupId)) throw Oops.Oh("加盟商id不能为空");
            var res = await _db.Queryable<BPA_GoodsUint>().Where(x => x.GroupId == groupId).Select<dynamic>(x => new

            {
                id = x.Id,
                name = x.Name
            }).ToListAsync();
            return res;
        }
        /// <summary>
        /// 添加商品单位
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> AddGoodsUint(GoodsUintDto dto)
        {
            var productCode = _db.Queryable<BPA_GoodsUint>().Where(x => x.Name == dto.Name).ToList();
            if (productCode.Count() > 0)
            {
                throw Oops.Oh($"{dto.Name}已存在，添加失败！");
            }
            try
            {
                BPA_GoodsUint bPA_Product = new BPA_GoodsUint();
                bPA_Product.Name = dto.Name;
                var res = await _db.Insertable(bPA_Product).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }
        }
        /// <summary>
        /// 商品配方
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<PageUtil> GetGoodsBomPageAsync(GoodsBomQueryDto dto)
        {
            RefAsync<int> total = 0;
            var res = await _db.Queryable<BPA_Bom, BPA_BomAttributeValueRe>((a, b) => new JoinQueryInfos(
                      JoinType.Left, a.Id == b.BoomId
                  ))
                  .WhereIF(!string.IsNullOrWhiteSpace(dto.GoodsId), (a, b) => b.GoodsId == dto.GoodsId)
                .WhereIF(!string.IsNullOrWhiteSpace(dto.Name), (a, b) => a.Name.Contains(dto.Name))
                  .WhereIF(!string.IsNullOrWhiteSpace(dto.GoodsAttributeValue), (a, b) => b.GoodsAttributeValue.Contains(dto.GoodsAttributeValue))
                .Select((a, b) => new GoodsBomPageView
                {
                    Id = b.Id,
                    Name = a.Name,
                    BomId = b.BoomId,
                    IsMain = a.IsMain,
                    GoodsAttributeValueId = b.GoodsAttributeValueId,
                    GoodsAttributeValue = b.GoodsAttributeValue,
                })
                .ToPageListAsync(dto.Current, dto.PageSize, total);
            PageUtil util = new PageUtil()
            {
                Total = total,
                Data = res
            };
            return util;
        }
        /// <summary>
        /// 删除商品配方
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        public async Task<bool> BatchDelGoodsBomAsync(string Ids)
        {
            // var GoodsBom=_db.Queryable<BPA_GoodsBom>().Where(x => x.Id == Ids).First();
            var model = _db.Queryable<BPA_BomAttributeValueRe>().Where(x => x.Id == Ids).First();
            if (model == null) throw Oops.Oh("配方不存在");
            _db.Deleteable<BPA_GoodsBom>().Where(x => x.Goods_Id == model.GoodsId && x.BomId == model.BoomId).ExecuteCommand();
            // 查询数据库中是否存在未删除的商品
            var res = _db.Deleteable<BPA_BomAttributeValueRe>().In(Ids).ExecuteCommand();
            return res > 0;
        }
        /// <summary>
        /// 添加商品配方
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> AddGoodsBomAttribute(GoodsBomAttributeDto dto)
        {
            _db.Ado.BeginTran();
            try
            {
                if (dto.Type == "add")
                {
                    var sortMax = _db.Queryable<BPA_Bom>().Max(x => x.Sort);
                    BPA_Bom newbPA_BOM = new BPA_Bom
                    {
                        Name = dto.BomName,
                        Code = GetNumber2(),
                        IsMain = dto.BomType == "1" ? true : false,
                        Sort = sortMax + 1,
                        Id = Guid.NewGuid().ToString(),
                        CreateAt = DateTime.Now,
                        Status = CommonStatus.ENABLE,
                    };
                    //添加配方
                    var res = _db.Insertable(newbPA_BOM).CallEntityMethod(m => m.Create()).ExecuteCommand();
                    //添加配方分类
                    var list = dto.BomtypeList.Select(item => new BPA_BomTypeInfo() { BomId = newbPA_BOM.Id, BomTypeId = item }).ToList();
                    _db.Insertable(list).CallEntityMethod(m => m.Create()).ExecuteCommand();

                    BPA_GoodsBom bom = new BPA_GoodsBom();
                    bom.Goods_Id = dto.GoodsId;
                    bom.BomId = newbPA_BOM.Id;
                    bom.Status = CommonStatus.ENABLE;
                    //添加商品配方关系
                    _db.Insertable(bom).CallEntityMethod(m => m.Create()).ExecuteCommand();
                    List<BPA_BomEntry> BOMEntryList = new();
                    if (dto.Mate.Count > 0)
                    {
                        for (int i = 0; i < dto.Mate.Count; i++)
                        {
                            BPA_BomEntry newbPA_BOMEnty = new BPA_BomEntry
                            {
                                BatchingId = dto.Mate[i].batchingId,
                                //BatchingKey = dto.Mate[i].AutoKey,
                                BomQty = dto.Mate[i].dosage,
                                BomId = newbPA_BOM.Id,
                                Status = CommonStatus.ENABLE,
                                IsReplace = false,
                            };
                            BOMEntryList.Add(newbPA_BOMEnty);
                        }
                    }
                    //添加配方物料关系
                    _db.Insertable(BOMEntryList).CallEntityMethod(m => m.Create()).ExecuteCommand();
                    BPA_BomAttributeValueRe bPA_BomAttributeValueRe = new();
                    bPA_BomAttributeValueRe.Id = Guid.NewGuid().ToString();
                    bPA_BomAttributeValueRe.GoodsId = dto.GoodsId;
                    bPA_BomAttributeValueRe.BoomId = newbPA_BOM.Id;
                    bPA_BomAttributeValueRe.GoodsAttributeValueId = string.Join(",", dto.Shuxing);
                    bPA_BomAttributeValueRe.GoodsAttributeValue = dto.GoodsAttributeValue;
                    //添加商品配方属性关系
                    _db.Insertable(bPA_BomAttributeValueRe).ExecuteCommand();
                }
                else
                {
                    if (string.IsNullOrWhiteSpace(dto.BomId)) throw Oops.Oh("请选择配方");
                    if (!string.IsNullOrWhiteSpace(dto.BomId))
                    {
                        var check = _db.Queryable<BPA_GoodsBom>().Where(x => x.Goods_Id == dto.GoodsId && x.BomId == dto.BomId).Any();
                        if (!check)
                        {
                            BPA_GoodsBom bom = new BPA_GoodsBom();
                            bom.Goods_Id = dto.GoodsId;
                            bom.BomId = dto.BomId;
                            bom.Status = CommonStatus.ENABLE;
                            //添加商品配方关系
                            _db.Insertable(bom).CallEntityMethod(m => m.Create()).ExecuteCommand();
                        }
                        string vid = string.Join(",", dto.Shuxing);
                        var checkvid = _db.Queryable<BPA_BomAttributeValueRe>().Where(x => x.GoodsId == dto.GoodsId && x.BoomId == dto.BomId && x.GoodsAttributeValueId == vid).Any();
                        if (checkvid) throw Oops.Oh("该配方已经存在");
                        //添加商品配方属性关系
                        BPA_BomAttributeValueRe bPA_BomAttributeValueRe = new();
                        bPA_BomAttributeValueRe.Id = Guid.NewGuid().ToString();
                        bPA_BomAttributeValueRe.GoodsId = dto.GoodsId;
                        bPA_BomAttributeValueRe.BoomId = dto.BomId;
                        bPA_BomAttributeValueRe.GoodsAttributeValueId = vid;
                        bPA_BomAttributeValueRe.GoodsAttributeValue = dto.GoodsAttributeValue;
                        _db.Insertable(bPA_BomAttributeValueRe).ExecuteCommand();
                    }

                }
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh("添加失败，失败信息：" + e.Message);
            }
        }
        public async Task<bool> AddGoodsBom(GoodsBomDto dto)
        {
            var res = false;
            List<BPA_GoodsBom> list = new();
            if (dto.BomId.Count() > 0)
            {
                for (int i = 0; i < dto.BomId.Count; i++)
                {
                    BPA_GoodsBom goodsBom = new();
                    goodsBom.BomId = dto.BomId[i];
                    goodsBom.Goods_Id = dto.GoodsId;
                    goodsBom.Status = CommonStatus.ENABLE;
                    list.Add(goodsBom);
                }
            }
            if (list.Count > 0)
            {
                res = await _db.Insertable(list).CallEntityMethod(m => m.Create()).ExecuteCommandAsync() > 0;
            }
            return res;
        }
        /// <summary>
        /// 根据id查询商品信息
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<GoodsInfoBaseView> GetGoods(string id)
        {
            var res = await _db.Queryable<BPA_GoodsInfo>()
                .Where((a) => a.Id == id)
                .OrderBy(a => a.Sort, OrderByType.Asc)
                .Select((a) => new GoodsInfoBaseView
                {
                    Id = a.Id,
                    Code = a.Code,
                    Name = a.Name,
                    Price = a.Price,
                    ImgUrl = a.ImgUrl,
                    Status = a.Status,
                    GoodsTypeId = a.GoodsTypeId,
                    GoodsTypeName = SqlFunc.Subqueryable<BPA_GoodsType>().Where(s => s.Id == a.GoodsTypeId).Select(s => s.Name),
                    Remark = a.Descritption,
                    // CreateAt = a.CreateAt,
                    GoodsUintId = a.GoodsUintId,
                    IsAttrubute = a.IsAttrubute,
                    GoodsUintName = SqlFunc.Subqueryable<BPA_GoodsUint>().Where(s => s.Id == a.GoodsUintId).Select(s => s.Name),
                    //GoodsAttributeList = new List<GoodsAttributeList>()
                }).FirstAsync();
            if (res != null)
            {
                if (!res.IsAttrubute)
                {
                    res.GoodsAttributeList = await _goodsAttributeService.GetByNameAttribute("默认属性");
                }
                else
                {
                    res.GoodsAttributeList = await _goodsAttributeService.GetByGoodsIdAttribute(res.Id);
                }
            }
            return res;
        }
        /// <summary>
        /// 根据ids获取商品信息
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        public async Task<List<GoodsInfoView>> GetGoodsWithIds(List<string> ids)
        {
            return await _db.Queryable<BPA_GoodsInfo>().Where(t => ids.Contains(t.Id)).OrderBy(a => a.Sort).Select(a => new GoodsInfoView { Id = a.Id, Name = a.Name }).ToListAsync();
        }
        public async Task<bool> ExportGood(IFormFile file)
        {
            bool isReplace = true;
            try
            {

                //var groupId = string.IsNullOrWhiteSpace(CurrentUser.TenantId)
                //? App.User?.FindFirst(ClaimConst.GroupId)?.Value
                // : CurrentUser.TenantId;
                var userId = App.User?.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
                if (string.IsNullOrEmpty(userId))
                {
                    userId = "";
                }
                var source = file.OpenReadStream();
                var mapper = new Mapper(source);
                _db.Ado.BeginTran();
                //添加 商品_物料信息
                #region 添加 商品_物料信息

                //1.查询并且添加类型
                var nonLiquid = await _db.Queryable<BPA_BatchingType>()
                    .Where(x => x.Name == "非液体").FirstAsync();
                if (nonLiquid == null)
                {
                    nonLiquid = new BPA_BatchingType()
                    {
                        Name = "非液体",
                    };
                    await _db.Insertable(nonLiquid).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                }
                var liquid = await _db.Queryable<BPA_BatchingType>()
                    .Where(x => x.Name == "液体料").FirstAsync();
                if (liquid == null)
                {
                    liquid = new BPA_BatchingType
                    {
                        Name = "液体料",
                    };
                    await _db.Insertable(liquid).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                }

                //单位
                var unit = await _db.Queryable<BPA_BatchingUint>().Where(x => x.Name == "默认单位").FirstAsync();
                if (unit == null)
                {
                    unit = new BPA_BatchingUint
                    {
                        Name = "默认单位",
                    };
                    await _db.Insertable(unit).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                }

                //3.物料
                ReadExcel<BatchingModel> readGoodBaseModelExcel = new();
                var GoodBaseModellist = readGoodBaseModelExcel.ExcelToList(mapper, "商品_物料信息");
                List<BPA_Batching> batchingList = GoodBaseModellist.Select(t => new BPA_Batching()
                {
                    Name = t.BatchingName,
                    Code = GetNumber2(),
                    StockUint = unit.Id,
                }).ToList();

                var batchingNameList = batchingList.Select(x => x.Name).ToList();

                var batchingListrDb = _db.Queryable<BPA_Batching>().Where(x => batchingNameList.Contains(x.Name))
                    .ToList();

                //替换
                if (isReplace)
                {

                    foreach (var item2 in batchingList)
                    {
                        var thisitem = batchingListrDb.FirstOrDefault(x => x.Name == item2.Name);
                        if (thisitem != null)
                        {
                            item2.Id = thisitem.Id;
                            item2.GroupId = thisitem.GroupId;
                            _db.Updateable(item2).ExecuteCommand();
                        }
                        else
                        {
                            _db.Insertable(item2).CallEntityMethod(m => m.Create()).ExecuteCommand();
                        }
                    }
                }
                else
                {
                    var batchingListrDbName = batchingListrDb.Select(x => x.Name).ToList();
                    batchingList = batchingList.Where(x => !batchingListrDbName.Contains(x.Name)).ToList();
                    await _db.Insertable(batchingList).ExecuteCommandAsync();
                }

                #endregion


                //添加 商品_基础信息
                #region 添加 商品_基础信息

                //1.查询大分类 和小分类
                var type = new BPA_GoodsType();
                var parentType = await _db.Queryable<BPA_GoodsType>()
                    .Where(x => x.Name == "默认分类" && x.GroupId == x.GroupId).FirstAsync();
                if (parentType == null)
                {
                    parentType = new BPA_GoodsType()
                    {
                        Name = "默认分类",
                        Remark = "",
                        Sort = 0,
                        Status = 0,
                        Pid = "0"
                    };

                    type = await _db.Insertable(parentType).CallEntityMethod(m => m.Create()).ExecuteReturnEntityAsync();
                    parentType = type;
                }
                //属性
                var goodsAttribute = await _db.Queryable<BPA_GoodsAttribute>()
                    .Where(x => x.AttributeName == "默认属性").FirstAsync();
                if (goodsAttribute == null)
                {
                    goodsAttribute = new BPA_GoodsAttribute()
                    {
                        AttributeName = "默认属性",
                        GoodsTypeId = parentType.Id,
                        Sort = 0,

                    };
                    _db.Insertable(goodsAttribute).CallEntityMethod(m => m.Create()).ExecuteCommand();
                }

                var goodsattributevalue = await _db.Queryable<BPA_GoodsAttributeValue>()
                    .FirstAsync(x => x.AttributeValue == "默认属性值" && x.GoodsAttributeId == goodsAttribute.Id);
                if (goodsattributevalue == null)
                {
                    goodsattributevalue = new BPA_GoodsAttributeValue()
                    {
                        AttributeValue = "默认属性值",
                        GoodsAttributeId = goodsAttribute.Id,
                        Sort = 1,
                    };
                    _db.Insertable(goodsattributevalue).CallEntityMethod(m => m.Create()).ExecuteCommand();
                }


                ReadExcel<GoodModel> readGoodModelExcel = new();
                var oodModellist = readGoodModelExcel.ExcelToList(mapper, "商品_基础信息");
                List<BPA_GoodsInfo> goodsInfo = new List<BPA_GoodsInfo>();
                int index = 1;
                foreach (var item in oodModellist.Where(x => !string.IsNullOrEmpty(x.Name)))
                {
                    //单位
                    var goodsUint = await _db.Queryable<BPA_GoodsUint>().Where(x => x.Name == "默认单位").FirstAsync();
                    if (goodsUint == null)
                    {
                        goodsUint = new BPA_GoodsUint()
                        {
                            Name = "默认单位",
                            Status = 0,
                        };
                        await _db.Insertable(goodsUint).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                    }
                    Console.WriteLine("数据解析完成，开始解析图片");
                    var images = await ExcelToDatatable(file.OpenReadStream(), Path.GetExtension(file.FileName), "商品_基础信息");
                    var imagePath = images.FirstOrDefault(x => x.Index == index);
                    goodsInfo.Add(new BPA_GoodsInfo()
                    {
                        Code = GetNumber2(),
                        CreateAt = DateTime.Now,
                        CreateBy = userId,
                        DefaultMate = null,
                        Design = "默认规格",
                        ForeignKeyRe = null,
                        GoodsUintId = goodsUint.Id,
                        Descritption = "",
                        ImgMode = 0,
                        ImgUrl = imagePath?.ImgesPath,
                        Name = item.Name,
                        Sort = 0,
                        GoodsTypeId = parentType.Id,
                        Price = 0,
                        IsAttrubute = false,
                        IsWeigh = false,
                        Key = null,
                        Status = 0,
                    });
                    index++;
                }

                var ngoodsInfoName = goodsInfo.Select(x => x.Name).ToList();
                var goodsInfoListDb = _db.Queryable<BPA_GoodsInfo>().Where(x => ngoodsInfoName.Contains(x.Name)).ToList();
                //替换
                if (isReplace)
                {

                    foreach (var item2 in goodsInfo)
                    {
                        var thisitem = goodsInfoListDb.FirstOrDefault(x => x.Name == item2.Name);

                        if (thisitem != null)
                        {
                            item2.Id = thisitem.Id;
                            item2.AutoKey = thisitem.AutoKey;
                            item2.GroupId = thisitem.GroupId;
                            _db.Updateable(item2).ExecuteCommand();
                        }
                        else
                        {
                            await _db.Insertable(item2).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                        }
                    }
                }
                else
                {
                    var goodsInfoListDbName = goodsInfoListDb.Select(x => x.Name).ToList();
                    goodsInfo = goodsInfo.Where(x => !goodsInfoListDbName.Contains(x.Name)).ToList();
                    await _db.Insertable(goodsInfo).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                }




                #endregion

                //添加设备商品仓位
                #region 添加设备商品仓位

                ReadExcel<DeviceGoodsPositionModel> deviceGoodsPositionModelExcel = new();
                var deviceGoodsPosition = deviceGoodsPositionModelExcel.ExcelToList(mapper, "设备物料仓位");

                var _deviceNames = deviceGoodsPosition.Select(x => x.DeviceName).ToList();
                var _goodsName = deviceGoodsPosition.Select(x => x.GoodsName).Distinct().ToList();

                var _deviceList = _db.Queryable<BPA_DeviceInfo>()
                    .Where(x => _deviceNames.Contains(x.DeviceName))
                    .ToList();

                var _goodsNameList = _db.Queryable<BPA_GoodsInfo>()
                    .Where(x => _goodsName.Contains(x.Name))
                    .ToList();

                var deviceGoodsPositionList = new List<BPA_DeviceGoodsPosition>();

                foreach (var item in deviceGoodsPosition)
                {
                    var thisDevice = _deviceList.FirstOrDefault(x => x.DeviceName == item.DeviceName);
                    var thisGoods = _goodsNameList.FirstOrDefault(x => x.Name == item.GoodsName);

                    deviceGoodsPositionList.Add(new BPA_DeviceGoodsPosition()
                    {
                        DeviceId = thisDevice?.Id,
                        GoodsId = thisGoods?.Id,
                        Id = Guid.NewGuid().ToString(),
                        Position1 = item.Position1,
                        Position2 = item.Position2,
                        Position3 = item.Position3,
                        Position4 = item.Position4,
                    });
                }

                var deviceGoodsPositionListDb = _db.Queryable<BPA_DeviceGoodsPosition>().ToList();
                foreach (var item in deviceGoodsPositionList)
                {
                    var dGPDb = deviceGoodsPositionListDb.FirstOrDefault(x => x.DeviceId == item.DeviceId && x.GoodsId == x.GoodsId);
                    if (dGPDb != null)
                    {
                        item.Id = dGPDb.Id;
                        await _db.Updateable(item).ExecuteCommandAsync();
                    }
                    else
                    {
                        await _db.Insertable(item).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                    }

                }

                #endregion


                //添加 功能_{商品名称}
                var tableName = ExcelToShtteName(file.OpenReadStream(), Path.GetExtension(file.FileName));

                #region 添加 工艺_{商品名称}

                foreach (var name in tableName)
                {
                    var nameList = name.Split('_');

                    //查询设备
                    var name1 = nameList[1];
                    var name2 = nameList[2];
                    var device = await _db.Queryable<BPA_DeviceInfo>().FirstAsync(x => x.DeviceName == name1);
                    if (device == null)
                    {
                        throw Oops.Oh(nameList + " 设备【" + nameList[1] + "】不存在");
                    }

                    ReadExcel<GoodTechnologyModel> goodTechnologyModelExcel = new();
                    var goodTechnologyModel = goodTechnologyModelExcel.ExcelToList(mapper, name);
                    var goodstechnologyactionDto = new List<GoodsTechnologyActionDto>();
                    var cName = "";
                    var info = new List<ActionJson>();
                    var namestr = nameList[2];
                    var goods = await _db.Queryable<BPA_GoodsInfo>().Where(x => x.Name == namestr).FirstAsync();
                    var goodsId = goods == null ? "" : goods.Id;
                    _db.Deleteable<BPA_GoodsTechnologyAction>().Where(x => x.GoodsId == goodsId).ExecuteCommand();
                    var goodTechnologyIndex = 0;
                    for (int i = 0; i < goodTechnologyModel.Count; i++)
                    {

                        var thisName = goodTechnologyModel[i].Name == null ? cName : goodTechnologyModel[i].Name;

                        var data = await _db.Queryable<BPA_Technology>()
                            .Where(x => x.Name == thisName && x.DeviceVersionId == device.ProductVersionId).FirstAsync();
                        if (data == null) throw Oops.Oh("该设备还没有设置工艺基础信息");
                        var data2 = await _db.Queryable<BPA_TechnologyAction>()
                           .Where(x => x.ActionName == goodTechnologyModel[i].Configuration && x.TechnologyId == data.Id).FirstAsync();
                        var batching = new BPA_Batching();
                        var IsBatch = goodTechnologyModel[i].Name != "液体料";

                        if (!string.IsNullOrEmpty(goodTechnologyModel[i].Name))
                        {

                            goodTechnologyIndex++;
                            cName = goodTechnologyModel[i].Name;
                            GoodsTechnologyActionDto item = new GoodsTechnologyActionDto()
                            {
                                ActionJson = "",
                                ChnologyId = data?.Id,
                                GoodsAttributeId = goodsattributevalue.Id,
                                GoodsId = goods?.Id,
                                IsBatch = IsBatch,
                                Sort = goodTechnologyIndex,
                                DeviceId = device?.Id,
                                StepName = goodTechnologyModel[i].Name,
                                Index = goodTechnologyIndex.ToString(),

                            };
                            info.Add(new ActionJson
                            {
                                chnologyId = data?.Id,
                                actionName = goodTechnologyModel[i].Configuration,
                                actionValue = goodTechnologyModel[i].Value,
                                technologyactionId = !IsBatch ? batching.Id : data2?.Id,
                                index = goodTechnologyIndex.ToString(),
                            });
                            goodstechnologyactionDto.Add(item);
                        }
                        else
                        {
                            info.Add(new ActionJson
                            {
                                chnologyId = data?.Id,
                                actionName = goodTechnologyModel[i].Configuration,
                                actionValue = goodTechnologyModel[i].Value,
                                technologyactionId = !IsBatch ? batching.Id : data2?.Id,
                                index = goodTechnologyIndex.ToString(),
                            }); ;
                        }
                    }
                    foreach (var thisItem in goodstechnologyactionDto)
                    {

                        var json = info.Where(x => x.chnologyId == thisItem.ChnologyId && x.index == thisItem.Index).ToList();

                        //if (thisItem.StepName == "液体料")
                        //{
                        //    foreach (var item in json)
                        //    {
                        //        var batching = await _db.Queryable<BPA_Batching>()
                        //            .FirstAsync(x => x.Batching_Name == item.actionName);
                        //        item.technologyactionId = batching?.Id;
                        //    }
                        //}
                        if (thisItem.StepName == "主料")
                        {
                            var v1 = json.FirstOrDefault(x => x.actionName == "主料名称");
                            var v2 = deviceGoodsPosition.FirstOrDefault(x => x.GoodsName == name2 && x.DeviceName == name1);
                            var jsondb = await _db.Queryable<BPA_TechnologyAction>()
                                .Where(x => x.TechnologyId == thisItem.ChnologyId)
                                .ToListAsync();

                            foreach (var item in from item in jsondb let insertableJsondb = json.FirstOrDefault(x => x.technologyactionId == item.Id) where insertableJsondb == null select item)
                            {
                                json.Add(new ActionJson()
                                {
                                    actionName = item.ActionName,
                                    actionValue = item.ActionName == "主料重量" ? "0" : item.ActionValue,
                                    chnologyId = item.TechnologyId,
                                    index = thisItem.Index,
                                    technologyactionId = item.Id

                                });
                            }

                            foreach (var thisitem2 in json)
                            {

                                if (thisitem2.actionName == "主料位置")
                                {
                                    if (v2 == null)
                                    {
                                        throw Oops.Oh(v1.actionValue + "物料没有确定位置");
                                    }

                                    thisitem2.actionValue = GetPositionMaterial(v2, v1.actionValue);
                                    if (string.IsNullOrEmpty(thisitem2.actionValue))
                                    {
                                        throw Oops.Oh(v1.actionValue + "物料没有确定位置");
                                    }

                                }
                            }
                        }

                        thisItem.ActionJson = JsonConvert.SerializeObject(json);
                    }

                    var insertableDta = goodstechnologyactionDto.Adapt<List<BPA_GoodsTechnologyAction>>();
                    await _db.Insertable(insertableDta).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                }

                #endregion


                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh("导入失败，失败信息：" + e.Message);
            }
        }
        /// <summary>
        /// 获取 物料仓位
        /// </summary>
        /// <returns></returns>
        private string GetPositionMaterial(DeviceGoodsPositionModel item, string goodsName)
        {

            var properties = item.GetType().GetProperties();
            foreach (var item2 in properties)
            {
                var value = item.GetType().GetProperty(item2.Name)?.GetValue(item);
                if (value == null)
                {
                    value = "";
                }
                if (value.ToString().Trim() == goodsName?.Trim())
                {
                    //ColumnAttribute attribute = (ColumnAttribute)item2.GetCustomAttribute(typeof(ColumnAttribute), false);
                    //return attribute?.Name;
                    if (item2.IsDefined(typeof(Npoi.Mapper.Attributes.ColumnAttribute), false))
                    {
                        Npoi.Mapper.Attributes.ColumnAttribute attribute = (Npoi.Mapper.Attributes.ColumnAttribute)item2.GetCustomAttribute(typeof(Npoi.Mapper.Attributes.ColumnAttribute), false);
                        return attribute.Name;
                    }
                }
            }
            return "";
        }
        /// <summary>
        /// 导出 设备工艺流程
        /// </summary>
        /// <returns></returns>
        public async Task<string> TechnologyTemplateExport(string deviceId)
        {
            var result = "";

            //查询设备
            var device = await _db.Queryable<BPA_DeviceInfo>().FirstAsync(x => x.Id == deviceId);

            //设备类型
            var dictData = await _db.Queryable<BPA_TechnologyExportRecode>().FirstAsync(x => x.DeviceVersionId == device.ProductVersionId);

            return dictData?.TemplatePath ?? "";
        }
        /// <summary>
        /// 设备 版本 工序模型导入
        /// </summary>
        /// <returns></returns>
        public async Task<bool> DeviceVersionTemplateImport(DeviceVersionTemplateImportInputDto inputDto)
        {
            var file = inputDto.file;
            try
            {
                _db.Ado.BeginTran();

                //if (inputDto.IsAddGoodsTemplate)
                //{

                //}
                //else
                //{

                var source = file.OpenReadStream();
                var mapper = new Mapper(source);
                var deviceVersion = await _db.Queryable<BPA_ProductVesion>().FirstAsync(x => x.ProductId == inputDto.DeviceClientType && x.Vesion == inputDto.Version);
                //添加 设备_烹饪工序模型
                #region  添加 设备_烹饪工序模型
                ReadExcel<GoodstechnologyModel> readGoodstechnologyExcel = new();
                var GoodstechnologyModellist = readGoodstechnologyExcel.ExcelToList(mapper);
                List<BPA_Technology> GoodsTechnologyList = new();
                List<BPA_TechnologyAction> GoodsTechnologyActionList = new();
                var GoodsTechnologyListDB = await _db.Queryable<BPA_Technology>()
                    .Where(x => x.DeviceVersionId == deviceVersion.Id).ToListAsync();
                string currennaid = "";
                string currennaname = "";
                for (int i = 0; i < GoodstechnologyModellist.Count; i++)
                {
                    var goodsTechnologyId = GoodsTechnologyListDB.FirstOrDefault(x => x.Name == GoodstechnologyModellist[i].Name);

                    if (!string.IsNullOrWhiteSpace(GoodstechnologyModellist[i].Name))
                    {

                        BPA_Technology item = new BPA_Technology()
                        {
                            Id = goodsTechnologyId == null ? Guid.NewGuid().ToString() : goodsTechnologyId.Id,
                            Name = GoodstechnologyModellist[i].Name,
                            DeviceVersionId = deviceVersion.Id
                        };
                        GoodsTechnologyList.Add(item);
                        currennaid = item.Id;
                        currennaname = item.Name;
                    }
                    var actvalueary = GoodstechnologyModellist[i].ActionValue?.Split('、');
                    if (string.IsNullOrEmpty(GoodstechnologyModellist[i].ActionType) && currennaname != "液体料")
                    {
                        throw Oops.Oh($"类型错误");
                    }
                    BPA_TechnologyAction technologyAction = new()
                    {
                        Id = Guid.NewGuid().ToString(),
                        TechnologyId = currennaid,
                        ActionName = GoodstechnologyModellist[i].ActionName,
                        ActionType = CharacterConversion(GoodstechnologyModellist[i].ActionType),
                        Sort = i,
                        TechnologyType = GoodstechnologyModellist[i].TechnologyType == "物料" ? TechnologyEnums.Batch : TechnologyEnums.Other,
                        CreateAt = DateTime.Now,
                        //ActionValue = GoodstechnologyModellist[i].ActionValue,
                    };
                    List<ActionValueModel> ActionValueModelList = new();
                    if (actvalueary != null)
                    {
                        foreach (var item in actvalueary)
                        {
                            ActionValueModelList.Add(new ActionValueModel() { actionValueName = item });
                        }
                        technologyAction.ActionValue = JsonConvert.SerializeObject(ActionValueModelList);
                    }
                    GoodsTechnologyActionList.Add(technologyAction);
                }

                //修改添加 主表
                foreach (var item in GoodsTechnologyList)
                {
                    var thisitem = GoodsTechnologyListDB.FirstOrDefault(x => x.Name == item.Name);

                    if (thisitem != null)
                    {
                        item.Id = thisitem.Id;
                        item.CreateAt = DateTime.Now;
                        _db.Updateable(item).ExecuteCommand();
                    }
                    else
                    {
                        //item.Id = Guid.NewGuid().ToString();
                        item.CreateAt = DateTime.Now;
                        await _db.Insertable(item).ExecuteCommandAsync();
                    }
                }

                //修改添从表
                GoodsTechnologyActionList = GoodsTechnologyActionList.Where(x => !string.IsNullOrEmpty(x.ActionName)).ToList();
                var goodstechnologyIds = GoodsTechnologyList.Select(x => x.Id).ToList();
                var GoodsTechnologyActionListDb = await _db.Queryable<BPA_TechnologyAction>()
                    .Where(x => goodstechnologyIds.Contains(x.TechnologyId)).ToListAsync();

                foreach (var item in GoodsTechnologyActionList)
                {
                    var thisitem = GoodsTechnologyActionListDb.FirstOrDefault(x => item.TechnologyId == x.TechnologyId && item.ActionName == x.ActionName);
                    if (thisitem != null)
                    {
                        item.Id = thisitem.Id;
                        item.CreateAt = DateTime.Now;
                        _db.Updateable(item).ExecuteCommand();
                    }
                    else
                    {
                        item.Id = Guid.NewGuid().ToString();
                        item.CreateAt = DateTime.Now;
                        await _db.Insertable(item).ExecuteCommandAsync();
                    }
                }
                #endregion
                var path = inputDto.AliyunUrl;
                var TechnologyExportRecode = _db.Queryable<BPA_TechnologyExportRecode>().Where(x => x.DeviceVersionId == inputDto.Id).First();
                if (TechnologyExportRecode != null)
                {
                    TechnologyExportRecode.CreateAt = DateTime.Now;
                    TechnologyExportRecode.TemplatePath = path;
                    var vvv = await _db.Updateable(TechnologyExportRecode).ExecuteCommandAsync();
                }
                else
                {

                    BPA_TechnologyExportRecode _TechnologyExportRecode = new();
                    _TechnologyExportRecode.Id = Guid.NewGuid().ToString();
                    _TechnologyExportRecode.CreateAt = DateTime.Now;
                    _TechnologyExportRecode.ProductId = inputDto.DeviceClientType;
                    _TechnologyExportRecode.DeviceVersionId = inputDto.Id;
                    _TechnologyExportRecode.TemplatePath = path;
                    await _db.Insertable(_TechnologyExportRecode).ExecuteCommandAsync();

                    //await _db.Updateable<BPA_TechnologyExportRecode>()
                    //    .SetColumns(it => it.TemplatePath == path)
                    //    .Where(it => it.Id == inputDto.Id)
                    //    .ExecuteCommandAsync();
                }

                //}
                _db.Ado.CommitTran();
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh($"Excel导入错误，" + e.Message);
            }
            return true;
        }

        /// <summary>
        /// 商品导入
        /// </summary>
        /// <param name="importDto"></param>
        /// <returns></returns>
        public async Task<bool> GoodsImport(GoodsInfoImportDto importDto)
        {
            var file = importDto.File;
            var aliyunHost = "https://bpa.oss-cn-chengdu.aliyuncs.com/";
            var userId = App.User?.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
            if (string.IsNullOrEmpty(userId))
            {
                userId = "";
            }
            var goodsInfo = await _db.Queryable<BPA_GoodsInfo>().ToListAsync();
            var goodsType = await _db.Queryable<BPA_GoodsType>().ToListAsync();
            var goodsUint = await _db.Queryable<BPA_GoodsUint>().ToListAsync();
            try
            {
                _db.Ado.BeginTran();
                var source = file.OpenReadStream();
                var mapper = new Mapper(source);
                var excelDto = new ReadExcel<GoodsImportDataDto>();
                var dataList = excelDto.ExcelToList(mapper);
                dataList.RemoveAll(t => t.Name == null);
                UpdateGoodsImportDto(dataList);
                var imageList = await GetImagesWithExcel(file.OpenReadStream());
                var newUnitList = new List<BPA_GoodsUint>();
                var i = 0;
                foreach (var data in dataList)
                {
                    if (data.Type == null)
                        throw Oops.Oh($"工艺分类不能为空！");
                    if (data.Name == null)
                        throw Oops.Oh($"商品名称不能为空！");
                    if (data.Uint == null)
                        throw Oops.Oh($"商品单位不能为空！");
                    var type = goodsType.FirstOrDefault(t => t.Name == data.Type);
                    if (type == null)
                        throw Oops.Oh($"{data.Name} 不存在分类：{data.Type}！");
                    var old = goodsInfo.FirstOrDefault(t => t.Name == data.Name && t.GoodsTypeId == type.Id);
                    if (old != null)
                        throw Oops.Oh($"{data.Name} 已存在该商品！");
                    var uintId = "";
                    var uintData = goodsUint.FirstOrDefault(t => t.Name == data.Uint);
                    var newUnitData = newUnitList.FirstOrDefault(t => t.Name == data.Uint);
                    if (uintData != null || newUnitData != null)
                        uintId = uintData != null ? uintData.Id : newUnitData.Id;
                    else
                    {
                        uintId = Guid.NewGuid().ToString();
                        var newUint = new BPA_GoodsUint
                        {
                            Id = uintId,
                            Name = data.Uint,
                            CreateBy = userId
                        };
                        newUnitList.Add(newUint);
                        await _db.Insertable(newUint).ExecuteCommandAsync();
                    }
                    var imgUrl = data.ImgUrl != null ? data.ImgUrl.ToString() : "";
                    if (!imgUrl.Contains("=DISPIMG"))
                        imgUrl = null;
                    else
                    {
                        imgUrl = aliyunHost + imageList[i];
                        i++;
                    }
                    var newValue = new BPA_GoodsInfo
                    {
                        Id = Guid.NewGuid().ToString(),
                        Name = data.Name,
                        GoodsTypeId = type.Id,
                        Descritption = data.Descritption,
                        ImgUrl = imgUrl,
                        Code = GetNumber2(8),
                        //Price = data.Price,
                        CreateBy = userId,
                        GoodsUintId = uintId,
                        IsWeigh = data.IsWeigh == "是",
                        IsAttrubute = true
                    };
                    await _db.Insertable(newValue).ExecuteCommandAsync();
                }
                _db.Ado.CommitTran();
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh($"Excel导入错误，" + e.Message);
            }
            return true;
        }

        public List<GoodsImportDataDto> UpdateGoodsImportDto(List<GoodsImportDataDto> list)
        {
            foreach (var item in list)
            {
                item.Name = item.Name?.Trim();
                item.Type = item.Type?.Trim();
                item.Uint = item.Uint?.Trim();
                item.IsWeigh = item.IsWeigh?.Trim();
                item.Descritption = item.Descritption?.Trim();
            }
            return list;
        }

        public async Task<List<string>> GetImagesWithExcel(Stream stream)
        {
            try
            {
                var result = new List<string>();
                IWorkbook workbook;
                workbook = new XSSFWorkbook(stream);
                var pictures = workbook.GetAllPictures();
                foreach (XSSFPictureData pic in pictures)
                {
                    using var ms = new MemoryStream(pic.Data);
                    var suffix = pic.SuggestFileExtension();
                    var fileName = Guid.NewGuid().ToString();
                    IFormFile formFile = new FormFile(ms, 0, ms.Length, "name", $"{fileName}.{suffix}");
                    result.Add(await _aliyunOssService.UpFileAsync(formFile));
                }
                return result;
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
        }

        /// <summary>
        /// 商品工艺导入
        /// </summary>
        /// <param name="importDto"></param>
        /// <returns></returns>
        public async Task<bool> GoodsTechnologyImport(GoodsTechnologyImportDto importDto)
        {
            string pattern = @"(?<part1>[^,(]+),(?<part2>\([^)]+\)),(?<part3>[^,]+)";
            var file = importDto.File;
            var userId = App.User?.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
            if (string.IsNullOrEmpty(userId))
            {
                userId = "";
            }
            var goodsInfoList = await _db.Queryable<BPA_GoodsInfo>().ToListAsync();
            var deviceList = await _db.Queryable<BPA_DeviceInfo>().ToListAsync();
            var technologyList = await _db.Queryable<BPA_Technology>().ToListAsync();
            var technologyActionList = await _db.Queryable<BPA_TechnologyAction>().ToListAsync();
            var wareHouseTemplateList = await _db.Queryable<BPA_WarehouseTemplate>().OrderBy(a => a.Id).ToListAsync();
            var batchList = await _db.Queryable<BPA_Batching>().Select(a => new BatchingView() { Id = a.Id, Name = a.Name }).ToListAsync();
            try
            {
                _db.Ado.BeginTran();
                var sheetNameList = GetSheetNameList(file.OpenReadStream());
                foreach (var sheetName in sheetNameList)
                {
                    var source = file.OpenReadStream();
                    var mapper = new Mapper(source);
                    var objModel = new ReadExcel<object>();
                    var objList = objModel.ExcelToList(mapper, sheetName);
                    var modelList = UpdateObjectToModel(objList);
                    modelList.RemoveAll(t => t.StepName == "主料(例)" && t.ActionName == "主料名称(例)" && t.ActionValue == "水(例)" && t.Index == "1(例)");
                    modelList.RemoveAll(t => t.StepName == "主料(例)" && t.ActionName == "主料位置(例)" && t.ActionValue == "1号位(例)" && t.Index == "2(例)");
                    modelList.RemoveAll(t => t.StepName == null && t.ActionName == null && t.ActionValue == null && t.Index == null);
                    modelList.RemoveAll(t => t.StepName == "工序" && t.ActionName == "工序参数" && t.ActionValue == "工序参数值" && t.Index == "值序号");
                    FillStepName(modelList);
                    var modelMatch = Regex.Match(modelList[0].StepName, pattern);
                    var deviceId = modelMatch.Groups["part3"].Value.Trim();
                    var deviceInfo = deviceList.FirstOrDefault(t => t.Id == deviceId);
                    wareHouseTemplateList = wareHouseTemplateList.Where(t => t.DeviceId == deviceId).ToList();
                    if (!wareHouseTemplateList.Any())
                        throw Oops.Oh($"{deviceInfo.DeviceName}不存在仓位模版，请先添加仓位模版！");
                    var goodId = modelMatch.Groups["part1"].Value.Trim();
                    await _db.Deleteable<BPA_GoodsTechnologyAction>().Where(x => x.GoodsId == goodId).ExecuteCommandAsync();
                    var stepName = "";
                    var actionName = "";
                    var ids = "";
                    var names = "";
                    var dataModel = new GoodsTechnologyDataModel { Values = new List<GoodsTechnologyValueModel>() };
                    var dataList = new List<GoodsTechnologyDataModel>();
                    var name = sheetName.Split(' ')[0];
                    foreach (var model in modelList)
                    {
                        if (model.StepName.Contains(goodId))
                        {
                            if (model.ActionValue == null && dataModel.Values.Count > 0)
                            {
                                dataList.Add(dataModel);
                                dataModel = new GoodsTechnologyDataModel();
                            }
                            dataModel.Ids = model.StepName;
                            dataModel.Names = model.ActionName;
                            dataModel.Values = new List<GoodsTechnologyValueModel>();
                            ids = model.StepName;
                            names = model.ActionName;
                            stepName = "";
                            actionName = "";
                        }
                        else
                        {
                            if (stepName == "" || stepName == model.StepName)
                            {
                                if (actionName == model.ActionName)
                                {
                                    dataList.Add(dataModel);
                                    dataModel = new GoodsTechnologyDataModel
                                    {
                                        Ids = ids,
                                        Names = names,
                                        Values = new List<GoodsTechnologyValueModel> { model }
                                    };
                                }
                                else
                                {
                                    dataModel.Values.Add(model);
                                    if (actionName == "")
                                        actionName = model.ActionName;
                                }
                            }
                            else
                            {
                                dataList.Add(dataModel);
                                dataModel = new GoodsTechnologyDataModel
                                {
                                    Ids = ids,
                                    Names = names,
                                    Values = new List<GoodsTechnologyValueModel> { model }
                                };
                                actionName = model.ActionName;
                            }
                            stepName = model.StepName;
                        }
                    }
                    if (dataModel.Values.Count > 0)
                        dataList.Add(dataModel);
                    var oldAtteibuteId = "";
                    var sort = 1;
                    var order = 0;
                    foreach (var data in dataList)
                    {
                        var attributeId = "";
                        Match idsMatch = Regex.Match(data.Ids, pattern);
                        if (idsMatch.Success)
                            attributeId = idsMatch.Groups["part2"].Value.Trim()[1..^1];
                        if (oldAtteibuteId == attributeId)
                            sort++;
                        else
                        {
                            order++;
                            sort = 1;
                        }
                        var groupList = data.Values.GroupBy(t => t.StepName).ToList();
                        foreach (var groupItem in groupList)
                        {
                            technologyList = technologyList.Where(t => t.DeviceVersionId == deviceList.FirstOrDefault(a => a.Id == deviceId)?.ProductVersionId).ToList();
                            technologyActionList = technologyActionList.Where(t => technologyList.Select(a => a.Id).Contains(t.TechnologyId)).ToList();
                            var technology = technologyList.FirstOrDefault(t => t.Name == groupItem.Key);
                            if (technology == null)
                                throw Oops.Oh($"{data.Names}的工序：{groupItem.Key} 不存在！");
                            var warehousrTemplateId = wareHouseTemplateList[0]?.Id;
                            var jsonList = new List<ActionJsonModel>();
                            var newActionNameList = new List<string>();
                            foreach (var item in groupItem)
                            {
                                newActionNameList.Add(item.ActionName);
                                var action = technologyActionList.FirstOrDefault(t => t.ActionName == item.ActionName);
                                if (action == null)
                                    throw Oops.Oh($"{data.Names}的工序参数：{item.ActionName} 不存在！");
                                if (action.ActionValue != null)
                                {
                                    var actionValueList = JsonConvert.DeserializeObject<List<ActionValueModel>>(action.ActionValue);
                                    var actionValues = actionValueList.Select(t => t.actionValueName).ToList();
                                    if (!actionValues.Contains(item.ActionValue))
                                        throw Oops.Oh($"{data.Names}的工序参数值：{item.ActionValue} 不存在！");
                                }
                                else if (action.TechnologyType == TechnologyEnums.Batch)
                                {
                                    var batchNameList = batchList.Select(t => t.Name).ToList();
                                    if (!batchNameList.Contains(item.ActionValue))
                                        throw Oops.Oh($"{data.Names}的物料名称：{item.ActionValue} 不存在！");
                                }
                                jsonList.Add(new ActionJsonModel
                                {
                                    actionName = item.ActionName,
                                    actionValue = item.ActionValue,
                                    technologyactionId = action.Id,
                                    index = item.Index
                                });
                            }
                            var oldActionNameList = technologyActionList.Where(t => t.TechnologyId == technology.Id).Select(t => t.ActionName).ToList();
                            var listsEqual = !newActionNameList.Except(oldActionNameList).Any() && newActionNameList.Count == oldActionNameList.Count;
                            if (!listsEqual)
                                throw Oops.Oh($"{data.Names}的{groupItem.Key}的工序参数值数量不一致！");
                            var actionJson = JsonConvert.SerializeObject(jsonList);
                            var newValue = new BPA_GoodsTechnologyAction
                            {
                                Id = Guid.NewGuid().ToString(),
                                StepName = groupItem.Key,
                                ChnologyId = technology.Id,
                                GoodsAttributeId = attributeId,
                                Sort = sort,
                                ActionJson = actionJson,
                                GoodsId = goodId,
                                DeviceId = deviceId,
                                CreateBy = userId,
                                WarehousrTemplateId = warehousrTemplateId,
                                Order = order
                            };
                            await _db.Insertable(newValue).ExecuteCommandAsync();
                            oldAtteibuteId = attributeId;
                        }
                    }
                }
                _db.Ado.CommitTran();
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh($"Excel导入错误，" + e.Message);
            }
            return true;
        }

        public List<GoodsTechnologyValueModel> UpdateObjectToModel(List<object> objList)
        {
            var result = new List<GoodsTechnologyValueModel>();
            foreach (var obj in objList)
            {
                var jsonObj = JsonConvert.SerializeObject(obj);
                var model = JsonConvert.DeserializeObject<GoodsTechnologyValueModel>(jsonObj);
                model.StepName = model.StepName?.Trim();
                model.ActionName = model.ActionName?.Trim();
                model.ActionValue = model.ActionValue?.Trim();
                model.Index = model.Index?.Trim();
                result.Add(model);
            }
            return result;
        }

        public List<GoodsTechnologyValueModel> FillStepName(List<GoodsTechnologyValueModel> list)
        {
            var stepName = "";
            foreach (var item in list)
            {
                if (item.StepName == null)
                    item.StepName = stepName;
                else
                    stepName = item.StepName;
            }
            return list;
        }

        /// <summary>
        /// 商品工艺模版导出
        /// </summary>
        /// <param name="exportDto"></param>
        /// <returns></returns>
        public async Task<string> GoodsTechnologyExport(GoodsTechnologyExportDto exportDto)
        {
            var goodsInfo = new List<BPA_GoodsInfo>();
            goodsInfo = exportDto.GoodsId.Count > 0
                ? await _db.Queryable<BPA_GoodsInfo>().Where(t => exportDto.GoodsId.Contains(t.Id)).ToListAsync()
                : await _db.Queryable<BPA_GoodsInfo, BPA_GoodsType>((a, b) => new JoinQueryInfos(JoinType.Left, a.GoodsTypeId == b.Id)).ToListAsync();
            var goodsTypeList = await _db.Queryable<BPA_GoodsType>().ToListAsync();
            var deviceInfo = await _db.Queryable<BPA_DeviceInfo>().FirstAsync(t => t.Id == exportDto.DeviceId);
            var goodsAttributeList = await _db.Queryable<BPA_GoodsAttribute>().ToListAsync();
            var goodsAttributeValueList = await _db.Queryable<BPA_GoodsAttributeValue>().Where(t => goodsAttributeList.Select(a => a.Id).Contains(t.GoodsAttributeId)).OrderBy(a => a.Sort).ToListAsync();
            IWorkbook workbook = new XSSFWorkbook();
            ICellStyle style = workbook.CreateCellStyle();
            IFont font = workbook.CreateFont();
            font.Boldweight = (short)FontBoldWeight.Bold;
            style.SetFont(font);
            style.FillForegroundColor = IndexedColors.Grey25Percent.Index;
            style.FillPattern = FillPattern.SolidForeground;
            style.VerticalAlignment = VerticalAlignment.Center;
            style.BorderBottom = BorderStyle.Thin;
            style.BottomBorderColor = IndexedColors.Black.Index;
            style.BorderLeft = BorderStyle.Thin;
            style.LeftBorderColor = IndexedColors.Black.Index;
            style.BorderRight = BorderStyle.Thin;
            style.RightBorderColor = IndexedColors.Black.Index;
            style.BorderTop = BorderStyle.Thin;
            style.TopBorderColor = IndexedColors.Black.Index;
            string[] charsToReplace = new string[] { "\\", "/", "?", "*", "[", "]" };
            foreach (var good in goodsInfo)
            {
                var type = goodsTypeList.FirstOrDefault(a => a.Id == good.GoodsTypeId);
                var name = good.Name + "_" + type.Name;
                foreach (string c in charsToReplace)
                {
                    name = name.Replace(c, " ");
                }
                ISheet sheet = workbook.CreateSheet(name);
                sheet.SetColumnWidth(0, 20 * 256);
                sheet.SetColumnWidth(1, 20 * 256);
                sheet.SetColumnWidth(2, 20 * 256);
                sheet.SetColumnWidth(3, 20 * 256);
                sheet.SetColumnWidth(4, 150 * 256);
                sheet.DefaultColumnWidth = 100 * 256;
                sheet.DefaultRowHeight = 30 * 20;
                IRow headerRow = sheet.CreateRow(0);
                headerRow.RowStyle = style;
                headerRow.CreateCell(0).SetCellValue("StepName");
                headerRow.CreateCell(1).SetCellValue("ActionName");
                headerRow.CreateCell(2).SetCellValue("ActionValue");
                headerRow.CreateCell(3).SetCellValue("Index");
                var ids = new Dictionary<string, List<string>>();
                var values = new Dictionary<string, List<string>>();
                var attrList = goodsAttributeList.Where(t => t.GoodsTypeId == good.GoodsTypeId).OrderBy(a => a.Sort).ToList();
                foreach (var attr in attrList)
                {
                    var attributeValueList = goodsAttributeValueList.Where(t => t.GoodsAttributeId == attr.Id).OrderBy(a => a.Sort).ToList();
                    ids.Add(attr.Id, attributeValueList.Select(t => t.Id).ToList());
                    values.Add(attr.AttributeName, attributeValueList.Select(a => a.AttributeValue).ToList());
                }
                var valueList = GenerateCombinations(values, false, good, deviceInfo);
                var idList = GenerateCombinations(ids, true, good, deviceInfo);
                var j = 0;
                for (int i = 0; i < int.MaxValue; i += 4)
                {
                    IRow dataRow = sheet.CreateRow(i + 1);
                    dataRow.RowStyle = style;
                    IRow dataRowTitle = sheet.CreateRow(i + 2);
                    dataRowTitle.RowStyle = style;
                    dataRow.CreateCell(0).SetCellValue(idList[j]);
                    dataRow.CreateCell(1).SetCellValue(valueList[j]);
                    dataRowTitle.CreateCell(0).SetCellValue("工序");
                    dataRowTitle.CreateCell(1).SetCellValue("工序参数");
                    dataRowTitle.CreateCell(2).SetCellValue("工序参数值");
                    dataRowTitle.CreateCell(3).SetCellValue("值序号");
                    j++;
                    if (idList.Count <= j)
                        break;
                }
                #region 填充两行示例数据和注意事项
                var dataRow1 = sheet.CreateRow(3);
                dataRow1.CreateCell(0).SetCellValue("主料(例)");
                dataRow1.CreateCell(1).SetCellValue("主料名称(例)");
                dataRow1.CreateCell(2).SetCellValue("水(例)");
                dataRow1.CreateCell(3).SetCellValue("1(例)");
                var dataRow2 = sheet.CreateRow(4);
                dataRow2.CreateCell(0).SetCellValue("主料(例)");
                dataRow2.CreateCell(1).SetCellValue("主料位置(例)");
                dataRow2.CreateCell(2).SetCellValue("1号位(例)");
                dataRow2.CreateCell(3).SetCellValue("2(例)");
                sheet.GetRow(1).CreateCell(4).SetCellValue($"！！！注意事项！！！\n(1)表格黑色背景部分请勿修改或删除；\n(2)导入商品工艺时请删除示例；\n(3)“工序(模型名称)”、“工序参数(模型配置)”、“工序参数值(输入值)”请查看设备工艺模版“{exportDto.FileName}”；");
                sheet.AddMergedRegion(new CellRangeAddress(1, 2, 4, 4));
                #endregion
            }
            var tempFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "TechnologyTemplate");
            if (!Directory.Exists(tempFolder))
                Directory.CreateDirectory(tempFolder);
            var time = DateTime.Now.ToString("yyyy年MM月dd日HH时mm分ss秒");
            var filePath = Path.Combine(tempFolder, $"{deviceInfo.DeviceName}工艺模版-{time}.xlsx");
            using (var stream = new FileStream(filePath, FileMode.Create, FileAccess.Write))
            {
                workbook.Write(stream);
            }
            var fileBase64 = Convert.ToBase64String(File.ReadAllBytes(filePath));
            return fileBase64;
        }

        /// <summary>
        /// 读取上传文件
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<List<GoodsChooseDto>> GetFileToChoose(GoodsInfoImportDto dto)
        {
            var result = new List<GoodsChooseDto>();
            var goodsList = await _db.Queryable<BPA_GoodsInfo>().ToListAsync();
            if(Path.GetExtension(dto.File.FileName) == ".zip")
            {
                using var archive = new ZipArchive(dto.File.OpenReadStream(), ZipArchiveMode.Read);
                foreach (var entry in archive.Entries)
                {
                    var sheetNameList = GetSheetNameList(entry.Open());
                    if (goodsList.Any(t => t.Name == sheetNameList[0]))
                        result.Add(new GoodsChooseDto
                        {
                            GoodsName = sheetNameList[0],
                            IsSkip = false
                        });
                }
            }
            else if(Path.GetExtension(dto.File.FileName) == ".xlsx")
            {
                var sheetNameList = GetSheetNameList(dto.File.OpenReadStream());
                if(sheetNameList.Any())
                {
                    if (goodsList.Any(t => t.Name == sheetNameList[0]))
                    {
                        result.Add(new GoodsChooseDto
                        {
                            GoodsName = sheetNameList[0],
                            IsSkip = false
                        });
                    }
                }
            }
            else
            {
                throw Oops.Oh("文件格式不正确");
            }
            return result;
        }

        /// <summary>
        /// 上传商品
        /// </summary>
        /// <param name="importDto"></param>
        /// <returns></returns>
        public async Task<bool> GoodsImport(GoodsImportDto importDto)
        {
            try
            {
                _db.Ado.BeginTran();
                var aliyunHost = "https://bpa.oss-cn-chengdu.aliyuncs.com/";
                var groupId = App.User.FindFirst(ClaimConst.GroupId)?.Value;
                var userId = App.User?.FindFirst(ClaimConst.CLAINM_USERID)?.Value;
                if (string.IsNullOrEmpty(userId))
                {
                    userId = "";
                }
                var file = importDto.File;
                var isSkipList = new List<GoodsImportJsonDto>();
                if (!string.IsNullOrWhiteSpace(importDto.Json))
                    isSkipList = JsonConvert.DeserializeObject<List<GoodsImportJsonDto>>(importDto.Json);
                var goodsList = await _db.Queryable<BPA_GoodsInfo>().ToListAsync();
                var goodsTypeList = await _db.Queryable<BPA_GoodsType>().ToListAsync();
                var goodsUintList = await _db.Queryable<BPA_GoodsUint>().ToListAsync();
                var goodAttrList = await _db.Queryable<BPA_GoodsAttribute>().ToListAsync();
                var goodAttrValueList = await _db.Queryable<BPA_GoodsAttributeValue>().ToListAsync();
                var deviceList = await _db.Queryable<BPA_DeviceInfo>().ToListAsync();
                var technologyList = await _db.Queryable<BPA_Technology>().ToListAsync();
                var technologyActionList = await _db.Queryable<BPA_TechnologyAction>().ToListAsync();
                var wareHouseTemplateList = await _db.Queryable<BPA_WarehouseTemplate>().OrderBy(a => a.Id).ToListAsync();
                var goodsTechnologyActionList = await _db.Queryable<BPA_GoodsTechnologyAction>().ToListAsync();
                var batchList = await _db.Queryable<BPA_Batching>().Select(a => new BatchingView() { Id = a.Id, Name = a.Name }).ToListAsync();
                var goodsDataAddList = new List<BPA_GoodsInfo>();
                var goodsDataUpdateList = new List<BPA_GoodsInfo>();
                var goodsTypeDataList = new List<BPA_GoodsType>();
                var goodsUintDataList = new List<BPA_GoodsUint>();
                var goodsAttrDataList = new List<BPA_GoodsAttribute>();
                var goodAttrValueDataList = new List<BPA_GoodsAttributeValue>();
                var goodsTechnologyActionDataList = new List<BPA_GoodsTechnologyAction>();
                var goodsTechnologyActionDataDeleteList = new List<BPA_GoodsTechnologyAction>();
                if (Path.GetExtension(file.FileName) == ".zip")
                {
                    using var archive = new ZipArchive(file.OpenReadStream(), ZipArchiveMode.Read, true, Encoding.GetEncoding("gb2312"));
                    foreach (var entry in archive.Entries)
                    {
                        if (entry.FullName.Contains("模型"))
                            continue;
                        using var source = entry.Open();
                        var memoryStream = new MemoryStream();
                        source.CopyTo(memoryStream);
                        memoryStream.Position = 0;
                        var sheetNameList = GetSheetNameList(memoryStream);
                        var isSkip = isSkipList.Count > 0 ? isSkipList.FirstOrDefault(t => t.key == sheetNameList[0])?.Value : true;
                        if (isSkip == false)
                            continue;
                        Mapper mapper;
                        using (var sourceCopy = entry.Open())
                        {
                            var memoryStreamCopy = new MemoryStream();
                            sourceCopy.CopyTo(memoryStreamCopy);
                            memoryStreamCopy.Position = 0;
                            mapper = new Mapper(memoryStreamCopy);
                        }
                        var goodsModelList = new ReadExcel<GoodsImportDataDto>().ExcelToList(mapper, sheetNameList[0]);
                        if (goodsModelList.Count <= 0 || goodsModelList[0].Name == null)
                            continue;
                        UpdateGoodsImportDataDto(goodsModelList);
                        var imageList = await GetImagesWithExcel(entry.Open());
                        var typeId = string.Empty;
                        var attrCount = 0;
                        var attrNameList = new List<string>();
                        var typeData = goodsTypeList.FirstOrDefault(t => t.Name == goodsModelList[0].Type);
                        if (typeData == null)
                        {
                            var typeInfo = goodsTypeDataList.FirstOrDefault(t => t.Name == goodsModelList[0].Type);
                            if (typeInfo == null)
                            {
                                typeInfo = new BPA_GoodsType
                                {
                                    Id = Guid.NewGuid().ToString(),
                                    Name = goodsModelList[0].Type,
                                    Pid = "0",
                                    CreateBy = userId
                                };
                                goodsTypeDataList.Add(typeInfo);
                            }
                            typeId = typeInfo.Id;
                        }
                        else
                        {
                            var attributeNameList = goodAttrList.Where(t => t.GoodsTypeId == typeData.Id).OrderBy(a => a.Sort).Select(b => b.AttributeName).ToList();
                            attrNameList.AddRange(attributeNameList);
                            attrCount = attributeNameList.Count;
                            typeId = typeData.Id;
                        }
                        var uintId = string.Empty;
                        var uintData = goodsUintList.FirstOrDefault(t => t.Name == goodsModelList[0].Uint);
                        if (uintData == null)
                        {
                            var uintInfo = goodsUintDataList.FirstOrDefault(t => t.Name == goodsModelList[0].Uint);
                            if (uintInfo == null)
                            {
                                uintInfo = new BPA_GoodsUint
                                {
                                    Id = Guid.NewGuid().ToString(),
                                    Name = goodsModelList[0].Uint,
                                    CreateBy = userId,
                                };
                                goodsUintDataList.Add(uintInfo);
                            }
                            uintId = uintInfo.Id;
                        }
                        else
                            uintId = uintData.Id;
                        var goodsInfo = goodsList.FirstOrDefault(t => t.Name == goodsModelList[0].Name);
                        if (goodsInfo == null)
                        {
                            goodsInfo = new BPA_GoodsInfo
                            {
                                Id = Guid.NewGuid().ToString(),
                                Name = goodsModelList[0].Name,
                                GoodsTypeId = typeId,
                                Descritption = goodsModelList[0].Descritption,
                                ImgUrl = imageList.Count > 0 ? aliyunHost + imageList[0] : null,
                                Code = GetNumber2(8),
                                Price = goodsModelList[0].Price,
                                CreateBy = userId,
                                GoodsUintId = uintId,
                                IsWeigh = goodsModelList[0].IsWeigh == "是",
                                IsAttrubute = true
                            };
                            goodsDataAddList.Add(goodsInfo);
                        }
                        else
                        {
                            goodsInfo.Name = goodsModelList[0].Name;
                            goodsInfo.GoodsTypeId = typeId;
                            goodsInfo.Descritption = goodsModelList[0].Descritption;
                            goodsInfo.ImgUrl = imageList.Count > 0 ? aliyunHost + imageList[0] : null;
                            goodsInfo.Price = goodsModelList[0].Price;
                            goodsInfo.GoodsUintId = uintId;
                            goodsInfo.IsWeigh = goodsModelList[0].IsWeigh == "是";
                            goodsDataUpdateList.Add(goodsInfo);
                        }
                        goodsTechnologyActionDataDeleteList.AddRange(goodsTechnologyActionList.Where(t => t.GoodsId == goodsInfo.Id).ToList());
                        var attrDataList = goodAttrList.Where(t => t.GoodsTypeId == typeId).ToList();
                        var attrValueDataList = goodAttrValueList.Where(t => attrDataList.Select(a => a.Id).Contains(t.GoodsAttributeId)).ToList();
                        var productVersionId = deviceList.FirstOrDefault(a => a.Id == importDto.DeviceId)?.ProductVersionId;
                        var attrValueSort = 0;
                        var allAttrValueIdList = new HashSet<string>();
                        var order = 1;
                        for (int i = 1; i < sheetNameList.Count; i++)
                        {
                            var technologyModelList = new ReadExcel<GoodsTechnologyValueModel>().ExcelToList(mapper, sheetNameList[i]);
                            var attrInfoList = GetAttributeData(technologyModelList);
                            if (attrNameList.Count > 0)
                                attrInfoList = attrNameList.Select(attrName => attrInfoList.FirstOrDefault(t => t.AttrName == attrName)).Where(attr => attr != null).ToList();
                            else
                                attrNameList = attrInfoList.Select(a => a.AttrName).ToList();
                            var technologyInfoList = GetTechnologyData(technologyModelList);
                            var technologyImportModelList = UpdateTechnologyImportModel(technologyInfoList);
                            var attrId = string.Empty;
                            var attrValueId = string.Empty;
                            var attrValueIdList = new List<string>();
                            var attrSort = 0;
                            foreach (var attr in attrInfoList)
                            {
                                var attrData = attrDataList.FirstOrDefault(t => t.AttributeName == attr.AttrName);
                                if (attrData == null)
                                {
                                    var attrInfo = goodsAttrDataList.FirstOrDefault(t => t.AttributeName == attr.AttrName);
                                    if (attrInfo == null)
                                    {
                                        attrInfo = new BPA_GoodsAttribute
                                        {
                                            Id = Guid.NewGuid().ToString(),
                                            AttributeName = attr.AttrName,
                                            GoodsTypeId = typeId,
                                            CreateBy = userId,
                                            Sort = attrSort + 1
                                        };
                                        goodsAttrDataList.Add(attrInfo);
                                    }
                                    attrId = attrInfo.Id;
                                }
                                else
                                    attrId = attrData.Id;
                                var attrValueData = attrValueDataList.FirstOrDefault(t => t.AttributeValue == attr.AttrValue);
                                if (attrValueData == null)
                                {
                                    var attrValueInfo = goodAttrValueDataList.FirstOrDefault(t => t.AttributeValue == attr.AttrValue);
                                    if (attrValueInfo == null)
                                    {
                                        attrValueInfo = new BPA_GoodsAttributeValue
                                        {
                                            Id = Guid.NewGuid().ToString(),
                                            GoodsAttributeId = attrId,
                                            AttributeValue = attr.AttrValue,
                                            CreateBy = userId,
                                            Sort = attrValueSort + 1
                                        };
                                        goodAttrValueDataList.Add(attrValueInfo);
                                    }
                                    attrValueId = attrValueInfo.Id;
                                    attrValueIdList.Add(attrValueId);
                                }
                                else
                                {
                                    attrValueId = attrValueData.Id;
                                    attrValueIdList.Add(attrValueId);
                                }
                            }
                            if (attrCount != 0)
                            {
                                if (attrCount != attrValueIdList.Count)
                                    throw Oops.Oh($"{entry.FullName} {sheetNameList[i]} 属性数量与已存在属性数量不一致");
                            }
                            else
                                attrCount = attrValueIdList.Count;
                            var attrValueIds = string.Join(",", attrValueIdList);
                            if (allAttrValueIdList.Contains(attrValueIds))
                                throw Oops.Oh($"{entry.FullName} {sheetNameList[i]} 已存在属性值组合");
                            allAttrValueIdList.Add(attrValueIds);
                            var sort = 1;
                            foreach (var item in technologyImportModelList)
                            {
                                var technology = technologyList.FirstOrDefault(t => t.Name == item.StepName && t.DeviceVersionId == productVersionId);
                                if (technology == null)
                                    throw Oops.Oh($"{entry.FullName} {sheetNameList[i]}的工序{item.StepName}不存在");
                                var jsonList = new List<ActionJsonModel>();
                                foreach (var child in item.Child)
                                {
                                    var action = technologyActionList.FirstOrDefault(t => t.ActionName == child.ActionName && t.TechnologyId == technology.Id);
                                    if (action == null)
                                        throw Oops.Oh($"{entry.FullName} {sheetNameList[i]}的工序参数{child.ActionName}不存在");
                                    if (action.ActionValue != null)
                                    {
                                        var actionValueList = JsonConvert.DeserializeObject<List<ActionValueModel>>(action.ActionValue);
                                        var actionValues = actionValueList.Select(t => t.actionValueName).ToList();
                                        if (!actionValues.Contains(child.ActionValue))
                                            throw Oops.Oh($"{entry.FullName} {sheetNameList[i]}的工序参数值{child.ActionName}不存在");
                                    }
                                    else if (action.TechnologyType == TechnologyEnums.Batch)
                                    {
                                        var batchNameList = batchList.Select(t => t.Name).ToList();
                                        if (!batchNameList.Contains(child.ActionValue))
                                            throw Oops.Oh($"{entry.FullName} {sheetNameList[i]}的物料名称{child.ActionValue}不存在");
                                    }
                                    jsonList.Add(new ActionJsonModel
                                    {
                                        actionName = child.ActionName,
                                        actionValue = child.ActionValue,
                                        technologyactionId = action.Id,
                                        index = child.Index
                                    });
                                }
                                var warehousrTemplateId = string.Empty;
                                var warehousrTemplate = wareHouseTemplateList.Where(t => t.DeviceId == importDto.DeviceId).OrderBy(a => a.Sort).FirstOrDefault();
                                if (warehousrTemplate != null)
                                    warehousrTemplateId = warehousrTemplate.Id;
                                goodsTechnologyActionDataList.Add(new BPA_GoodsTechnologyAction
                                {
                                    Id = Guid.NewGuid().ToString(),
                                    StepName = item.StepName,
                                    ChnologyId = technology.Id,
                                    GoodsAttributeId = string.Join(",", attrValueIdList),
                                    Sort = sort,
                                    ActionJson = JsonConvert.SerializeObject(jsonList),
                                    GoodsId = goodsInfo.Id,
                                    DeviceId = importDto.DeviceId,
                                    CreateBy = userId,
                                    WarehousrTemplateId = warehousrTemplateId,
                                    Order = order
                                });
                                sort++;
                            }
                            order++;
                        }
                    }
                }
                else
                {
                    var sheetNameList = GetSheetNameList(file.OpenReadStream());
                    var isSkip = isSkipList.Count > 0 ? isSkipList.FirstOrDefault(t => t.key == sheetNameList[0])?.Value : true;
                    if (isSkip == false)
                        return true;
                    Mapper mapper;
                    using (var sourceCopy = file.OpenReadStream())
                    {
                        var memoryStreamCopy = new MemoryStream();
                        sourceCopy.CopyTo(memoryStreamCopy);
                        memoryStreamCopy.Position = 0;
                        mapper = new Mapper(memoryStreamCopy);
                    }
                    var goodsModelList = new ReadExcel<GoodsImportDataDto>().ExcelToList(mapper, sheetNameList[0]);
                    if (goodsModelList.Count <= 0 || goodsModelList[0].Name == null)
                        return true;
                    UpdateGoodsImportDataDto(goodsModelList);
                    var imageList = await GetImagesWithExcel(file.OpenReadStream());
                    var typeId = string.Empty;
                    var attrCount = 0;
                    var attrNameList = new List<string>();
                    var typeData = goodsTypeList.FirstOrDefault(t => t.Name == goodsModelList[0].Type);
                    if (typeData == null)
                    {
                        var typeInfo = goodsTypeDataList.FirstOrDefault(t => t.Name == goodsModelList[0].Type);
                        if (typeInfo == null)
                        {
                            typeInfo = new BPA_GoodsType
                            {
                                Id = Guid.NewGuid().ToString(),
                                Name = goodsModelList[0].Type,
                                Pid = "0",
                                CreateBy = userId
                            };
                            goodsTypeDataList.Add(typeInfo);
                        }
                        typeId = typeInfo.Id;
                    }
                    else
                    {
                        var attributeNameList = goodAttrList.Where(t => t.GoodsTypeId == typeData.Id).OrderBy(a => a.Sort).Select(b => b.AttributeName).ToList();
                        attrNameList.AddRange(attributeNameList);
                        attrCount = attributeNameList.Count;
                        typeId = typeData.Id;
                    }
                    var uintId = string.Empty;
                    var uintData = goodsUintList.FirstOrDefault(t => t.Name == goodsModelList[0].Uint);
                    if (uintData == null)
                    {
                        var uintInfo = goodsUintDataList.FirstOrDefault(t => t.Name == goodsModelList[0].Uint);
                        if (uintInfo == null)
                        {
                            uintInfo = new BPA_GoodsUint
                            {
                                Id = Guid.NewGuid().ToString(),
                                Name = goodsModelList[0].Uint,
                                CreateBy = userId,
                            };
                            goodsUintDataList.Add(uintInfo);
                        }
                        uintId = uintInfo.Id;
                    }
                    else
                        uintId = uintData.Id;
                    var goodsInfo = goodsList.FirstOrDefault(t => t.Name == goodsModelList[0].Name);
                    if (goodsInfo == null)
                    {
                        goodsInfo = new BPA_GoodsInfo
                        {
                            Id = Guid.NewGuid().ToString(),
                            Name = goodsModelList[0].Name,
                            GoodsTypeId = typeId,
                            Descritption = goodsModelList[0].Descritption,
                            ImgUrl = imageList.Count > 0 ? aliyunHost + imageList[0] : null,
                            Code = GetNumber2(8),
                            Price = goodsModelList[0].Price,
                            CreateBy = userId,
                            GoodsUintId = uintId,
                            IsWeigh = goodsModelList[0].IsWeigh == "是",
                            IsAttrubute = true
                        };
                        goodsDataAddList.Add(goodsInfo);
                    }
                    else
                    {
                        goodsInfo.Name = goodsModelList[0].Name;
                        goodsInfo.GoodsTypeId = typeId;
                        goodsInfo.Descritption = goodsModelList[0].Descritption;
                        goodsInfo.ImgUrl = imageList.Count > 0 ? aliyunHost + imageList[0] : null;
                        goodsInfo.Price = goodsModelList[0].Price;
                        goodsInfo.GoodsUintId = uintId;
                        goodsInfo.IsWeigh = goodsModelList[0].IsWeigh == "是";
                        goodsDataUpdateList.Add(goodsInfo);
                    }
                    goodsTechnologyActionDataDeleteList.AddRange(goodsTechnologyActionList.Where(t => t.GoodsId == goodsInfo.Id).ToList());
                    var attrDataList = goodAttrList.Where(t => t.GoodsTypeId == typeId).ToList();
                    var attrValueDataList = goodAttrValueList.Where(t => attrDataList.Select(a => a.Id).Contains(t.GoodsAttributeId)).ToList();
                    var productVersionId = deviceList.FirstOrDefault(a => a.Id == importDto.DeviceId)?.ProductVersionId;
                    var attrValueSort = 0;
                    var allAttrValueIdList = new HashSet<string>();
                    var order = 1;
                    for (int i = 1; i < sheetNameList.Count; i++)
                    {
                        var technologyModelList = new ReadExcel<GoodsTechnologyValueModel>().ExcelToList(mapper, sheetNameList[i]);
                        var attrInfoList = GetAttributeData(technologyModelList);
                        if (attrNameList.Count > 0)
                            attrInfoList = attrNameList.Select(attrName => attrInfoList.FirstOrDefault(t => t.AttrName == attrName)).Where(attr => attr != null).ToList();
                        else
                            attrNameList = attrInfoList.Select(a => a.AttrName).ToList();
                        var technologyInfoList = GetTechnologyData(technologyModelList);
                        var technologyImportModelList = UpdateTechnologyImportModel(technologyInfoList);
                        var attrId = string.Empty;
                        var attrValueId = string.Empty;
                        var attrValueIdList = new List<string>();
                        var attrSort = 0;
                        foreach (var attr in attrInfoList)
                        {
                            var attrData = attrDataList.FirstOrDefault(t => t.AttributeName == attr.AttrName);
                            if (attrData == null)
                            {
                                var attrInfo = goodsAttrDataList.FirstOrDefault(t => t.AttributeName == attr.AttrName);
                                if (attrInfo == null)
                                {
                                    attrInfo = new BPA_GoodsAttribute
                                    {
                                        Id = Guid.NewGuid().ToString(),
                                        AttributeName = attr.AttrName,
                                        GoodsTypeId = typeId,
                                        CreateBy = userId,
                                        Sort = attrSort + 1
                                    };
                                    goodsAttrDataList.Add(attrInfo);
                                }
                                attrId = attrInfo.Id;
                            }
                            else
                                attrId = attrData.Id;
                            var attrValueData = attrValueDataList.FirstOrDefault(t => t.AttributeValue == attr.AttrValue);
                            if (attrValueData == null)
                            {
                                var attrValueInfo = goodAttrValueDataList.FirstOrDefault(t => t.AttributeValue == attr.AttrValue);
                                if (attrValueInfo == null)
                                {
                                    attrValueInfo = new BPA_GoodsAttributeValue
                                    {
                                        Id = Guid.NewGuid().ToString(),
                                        GoodsAttributeId = attrId,
                                        AttributeValue = attr.AttrValue,
                                        CreateBy = userId,
                                        Sort = attrValueSort + 1
                                    };
                                    goodAttrValueDataList.Add(attrValueInfo);
                                }
                                attrValueId = attrValueInfo.Id;
                                attrValueIdList.Add(attrValueId);
                            }
                            else
                            {
                                attrValueId = attrValueData.Id;
                                attrValueIdList.Add(attrValueId);
                            }
                        }
                        if (attrCount != 0)
                        {
                            if (attrCount != attrValueIdList.Count)
                                throw Oops.Oh($"{file.FileName} {sheetNameList[i]} 属性数量与已存在属性数量不一致");
                        }
                        else
                            attrCount = attrValueIdList.Count;
                        var attrValueIds = string.Join(",", attrValueIdList);
                        if (allAttrValueIdList.Contains(attrValueIds))
                            throw Oops.Oh($"{file.FileName} {sheetNameList[i]} 已存在属性值组合");
                        allAttrValueIdList.Add(attrValueIds);
                        var sort = 1;
                        foreach (var item in technologyImportModelList)
                        {
                            var technology = technologyList.FirstOrDefault(t => t.Name == item.StepName && t.DeviceVersionId == productVersionId);
                            if (technology == null)
                                throw Oops.Oh($"{file.FileName} {sheetNameList[i]}的工序{item.StepName}不存在");
                            var jsonList = new List<ActionJsonModel>();
                            foreach (var child in item.Child)
                            {
                                var action = technologyActionList.FirstOrDefault(t => t.ActionName == child.ActionName && t.TechnologyId == technology.Id);
                                if (action == null)
                                    throw Oops.Oh($"{file.FileName} {sheetNameList[i]}的工序参数{child.ActionName}不存在");
                                if (action.ActionValue != null)
                                {
                                    var actionValueList = JsonConvert.DeserializeObject<List<ActionValueModel>>(action.ActionValue);
                                    var actionValues = actionValueList.Select(t => t.actionValueName).ToList();
                                    if (!actionValues.Contains(child.ActionValue))
                                        throw Oops.Oh($"{file.FileName} {sheetNameList[i]}的工序参数值{child.ActionName}不存在");
                                }
                                else if (action.TechnologyType == TechnologyEnums.Batch)
                                {
                                    var batchNameList = batchList.Select(t => t.Name).ToList();
                                    if (!batchNameList.Contains(child.ActionValue))
                                        throw Oops.Oh($"{file.FileName} {sheetNameList[i]}的物料名称{child.ActionValue}不存在");
                                }
                                jsonList.Add(new ActionJsonModel
                                {
                                    actionName = child.ActionName,
                                    actionValue = child.ActionValue,
                                    technologyactionId = action.Id,
                                    index = child.Index
                                });
                            }
                            var warehousrTemplateId = string.Empty;
                            var warehousrTemplate = wareHouseTemplateList.Where(t => t.DeviceId == importDto.DeviceId).OrderBy(a => a.Sort).FirstOrDefault();
                            if (warehousrTemplate != null)
                                warehousrTemplateId = warehousrTemplate.Id;
                            goodsTechnologyActionDataList.Add(new BPA_GoodsTechnologyAction
                            {
                                Id = Guid.NewGuid().ToString(),
                                StepName = item.StepName,
                                ChnologyId = technology.Id,
                                GoodsAttributeId = string.Join(",", attrValueIdList),
                                Sort = sort,
                                ActionJson = JsonConvert.SerializeObject(jsonList),
                                GoodsId = goodsInfo.Id,
                                DeviceId = importDto.DeviceId,
                                CreateBy = userId,
                                WarehousrTemplateId = warehousrTemplateId,
                                Order = order
                            });
                            sort++;
                        }
                        order++;
                    }
                }
                await _db.Insertable(goodsDataAddList).ExecuteCommandAsync();
                await _db.Updateable(goodsDataUpdateList).ExecuteCommandAsync();
                await _db.Insertable(goodsTypeDataList).ExecuteCommandAsync();
                await _db.Insertable(goodsUintDataList).ExecuteCommandAsync();
                await _db.Insertable(goodsAttrDataList).ExecuteCommandAsync();
                await _db.Insertable(goodAttrValueDataList).ExecuteCommandAsync();
                await _db.Insertable(goodsTechnologyActionDataList).ExecuteCommandAsync();
                await _db.Deleteable(goodsTechnologyActionDataDeleteList).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                _db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        public List<GoodsImportDataDto> UpdateGoodsImportDataDto(List<GoodsImportDataDto> list)
        {
            list.RemoveAll(t => t.Name == null);
            list.RemoveAll(t => t.Name == "牛肉面(例)");
            foreach (var item in list)
            {
                item.Name = item.Name?.Trim();
                item.Type = item.Type?.Trim();
                item.Uint = item.Uint?.Trim();
                item.IsWeigh = item.IsWeigh?.Trim();
                item.Descritption = item.Descritption?.Trim();
            }
            return list;
        }

        public List<AttributeDataDto> GetAttributeData(List<GoodsTechnologyValueModel> list)
        {
            list.RemoveAll(t => t.StepName == null);
            list.RemoveAll(t => t.StepName == "份量(例)");
            list.RemoveAll(t => t.StepName == "口味(例)");
            list.RemoveAll(t => t.StepName == "主料(例)");
            var result = new List<AttributeDataDto>();
            foreach (var item in list)
            {
                if (item.StepName == "属性" || item.StepName == "")
                    continue;
                if (item.StepName == "工序")
                    break;
                result.Add(new AttributeDataDto
                {
                    AttrName = item.StepName.Trim(),
                    AttrValue = item.ActionName.Trim()
                });
            }
            return result;
        }

        public List<GoodsTechnologyValueModel> GetTechnologyData(List<GoodsTechnologyValueModel> list)
        {
            list.RemoveAll(t => t.StepName == null);
            list.RemoveAll(t => t.StepName == "份量(例)");
            list.RemoveAll(t => t.StepName == "口味(例)");
            list.RemoveAll(t => t.StepName == "主料(例)");
            var result = new List<GoodsTechnologyValueModel>();
            var isAdd = false;
            foreach (var item in list)
            {
                if (isAdd)
                {
                    result.Add(new GoodsTechnologyValueModel
                    {
                        StepName = item.StepName?.Trim(),
                        ActionName = item.ActionName?.Trim(),
                        ActionValue = item.ActionValue?.Trim(),
                        Index = item.Index?.Trim()
                    });
                }
                if (item.StepName == "工序")
                    isAdd = true;
            }
            var stepName = "";
            foreach (var item in result)
            {
                if (item.StepName == null)
                    item.StepName = stepName;
                else
                    stepName = item.StepName;
            }
            return result;
        }

        public List<TechnologyImportModel> UpdateTechnologyImportModel(List<GoodsTechnologyValueModel> list)
        {
            var result = new List<TechnologyImportModel>();
            var stepName = "";
            var actionName = "";
            var childList = new List<TechnologyImportChildModel>();
            foreach (var item in list)
            {
                if (stepName == "" && actionName == "")
                {
                    stepName = item.StepName;
                    actionName = item.ActionName;
                    childList.Add(new TechnologyImportChildModel
                    {
                        ActionName = item.ActionName,
                        ActionValue = item.ActionValue,
                        Index = item.Index
                    });
                }
                else if (stepName == item.StepName)
                {
                    if (actionName == item.ActionName)
                    {
                        result.Add(new TechnologyImportModel { StepName = stepName, Child = childList });
                        childList = new List<TechnologyImportChildModel>
                        {
                            new TechnologyImportChildModel
                            {
                                ActionName = item.ActionName,
                                ActionValue = item.ActionValue,
                                Index = item.Index
                            }
                        };
                    }
                    else
                    {
                        childList.Add(new TechnologyImportChildModel
                        {
                            ActionName = item.ActionName,
                            ActionValue = item.ActionValue,
                            Index = item.Index
                        });
                    }
                }
                else
                {
                    result.Add(new TechnologyImportModel { StepName = stepName, Child = childList });
                    childList = new List<TechnologyImportChildModel>
                    {
                        new TechnologyImportChildModel
                        {
                            ActionName = item.ActionName,
                            ActionValue = item.ActionValue,
                            Index = item.Index
                        }
                    };
                    stepName = item.StepName;
                    actionName = item.ActionName;
                }
            }
            result.Add(new TechnologyImportModel { StepName = stepName, Child = childList });
            return result;
        }

        /// <summary>
        /// 下载商品
        /// </summary>
        /// <param name="exportDto"></param>
        /// <returns></returns>
        public async Task<string> GoodsExport(GoodsExportDto exportDto)
        {
            var goodTypeList = await _db.Queryable<BPA_GoodsType>().ToListAsync();
            var goodUintList = await _db.Queryable<BPA_GoodsUint>().ToListAsync();
            var goodAttrList = await _db.Queryable<BPA_GoodsAttribute>().ToListAsync();
            var goodAttrValueList = await _db.Queryable<BPA_GoodsAttributeValue>().ToListAsync();
            var actionList = await _db.Queryable<BPA_GoodsTechnologyAction>().ToListAsync();
            var deviceInfo = await _db.Queryable<BPA_DeviceInfo>().FirstAsync(t => t.Id == exportDto.DeviceId);
            if (deviceInfo == null)
                throw Oops.Oh("该设备不存在！");
            var recode = await _db.Queryable<BPA_TechnologyExportRecode>().FirstAsync(x => x.DeviceVersionId == deviceInfo.ProductVersionId);
            if (recode == null)
                throw Oops.Oh($"该设备工艺模版不存在！");
            var recodeName = Path.GetFileName(recode.TemplatePath);
            var tempFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "GoodsTechnologyData");
            if (Directory.Exists(tempFolder))
                Directory.Delete(tempFolder, true);
            Directory.CreateDirectory(tempFolder);
            var zipFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "GoodsTechnologyData.zip");
            if (File.Exists(zipFolder))
                File.Delete(zipFolder);
            await DownloadRemoteFile(recode.TemplatePath, tempFolder, recodeName);
            var dic = new Dictionary<string, string> { { "groupId", App.User.FindFirst(ClaimConst.GroupId)?.Value } };
            var url = App.Configuration["baseurl"] + $"api/store/getbyIdstorelist_alm?id={deviceInfo.StopId}";
            var stopInfo = JsonConvert.DeserializeObject<ResultSAASManageDto<StoreDto>>(HttpHelper.HttpGet(url, null, dic, "application/json"));
            if (stopInfo?.data == null)
                throw Oops.Oh("店铺不存在！");
            var goodInfoList = exportDto.GoodId != null ? await _db.Queryable<BPA_GoodsInfo>().Where(t => t.Id == exportDto.GoodId).ToListAsync() : await _db.Queryable<BPA_GoodsInfo>().Where(t => exportDto.GoodsId.Contains(t.Id)).ToListAsync();
            if (goodInfoList.Count == 0)
            {
                var templateUrl = "https://bpa.oss-cn-chengdu.aliyuncs.com/hkerp/test/goods//商品模版.xlsx";
                var templateName = Path.GetFileName(templateUrl);
                await DownloadRemoteFile(templateUrl, tempFolder, templateName);
            }
            else
            {
                foreach (var goodInfo in goodInfoList)
                {
                    var goodType = goodTypeList.FirstOrDefault(t => t.Id == goodInfo.GoodsTypeId);
                    var goodUint = goodUintList.FirstOrDefault(t => t.Id == goodInfo.GoodsUintId);
                    IWorkbook workbook = new XSSFWorkbook();
                    ICellStyle titleStyle = workbook.CreateCellStyle();
                    IFont font = workbook.CreateFont();
                    UpdateTitleStyle(titleStyle, font);
                    ICellStyle dataStyle = workbook.CreateCellStyle();
                    UpdateDataStyle(dataStyle);
                    string[] charsToReplace = new string[] { "\\", "/", "?", "*", "[", "]" };
                    foreach (string c in charsToReplace)
                    {
                        goodInfo.Name = goodInfo.Name.Replace(c, "_");
                    }
                    ISheet goodSheet = workbook.CreateSheet(goodInfo.Name);
                    IRow headerRow = goodSheet.CreateRow(0);
                    headerRow.RowStyle = titleStyle;
                    headerRow.CreateCell(0).SetCellValue("商品名称");
                    headerRow.CreateCell(1).SetCellValue("工艺分类");
                    headerRow.CreateCell(2).SetCellValue("商品单位");
                    headerRow.CreateCell(3).SetCellValue("商品价格(¥)");
                    headerRow.CreateCell(4).SetCellValue("是否称重");
                    headerRow.CreateCell(5).SetCellValue("备注");
                    headerRow.CreateCell(6).SetCellValue("图片");
                    headerRow.CreateCell(7).SetCellValue($"！！！注意！！！\r\n(1)灰色标题栏请勿删除！\r\n(2)请删除带“(例)”的示例数据！\r\n(3)sheet名称请以商品名称命名，如：牛肉面！\r\n(4)图片请选择“嵌入到单元格”，请勿复制粘贴！");
                    headerRow.GetCell(7).CellStyle = titleStyle;
                    goodSheet.AddMergedRegion(new CellRangeAddress(0, 1, 7, 7));
                    IRow dataRow = goodSheet.CreateRow(1);
                    dataRow.RowStyle = dataStyle;
                    dataRow.CreateCell(0).SetCellValue(goodInfo.Name);
                    dataRow.CreateCell(1).SetCellValue(goodType?.Name);
                    dataRow.CreateCell(2).SetCellValue(goodUint?.Name);
                    dataRow.CreateCell(3).SetCellValue(goodInfo.Price.ToString());
                    dataRow.CreateCell(4).SetCellValue(goodInfo.IsWeigh ? "是" : "否");
                    dataRow.CreateCell(5).SetCellValue(goodInfo.Descritption);
                    if (!string.IsNullOrWhiteSpace(goodInfo.ImgUrl) && goodInfo.ImgUrl.Contains("http"))
                        await InsertImage(goodSheet, goodInfo.ImgUrl, 1, 6);
                    var actions = actionList.Where(t => t.GoodsId == goodInfo.Id && t.DeviceId == exportDto.DeviceId).OrderBy(a => a.Order).ThenBy(b => b.Sort).GroupBy(c => c.GoodsAttributeId);
                    foreach (var action in actions)
                    {
                        var attrIds = action.Key.Split(',');
                        var attrs = new List<AttrDto>();
                        foreach (var attrId in attrIds)
                        {
                            var a = goodAttrValueList.FirstOrDefault(t => t.Id == attrId);
                            if (a == null)
                                throw Oops.Oh($"{goodInfo.Name}的属性值不存在！");
                            var b = goodAttrList.FirstOrDefault(t => t.Id == a.GoodsAttributeId);
                            if (b == null)
                                throw Oops.Oh($"{goodInfo.Name}属性不存在！");
                            attrs.Add(new AttrDto { Attr = b.AttributeName, AttrValue = a.AttributeValue });
                        }
                        ISheet actionSheet = workbook.CreateSheet(goodInfo.Name + " " + string.Join('-', attrs.Select(a => a.AttrValue)));
                        IRow markRow = actionSheet.CreateRow(0);
                        markRow.RowStyle = titleStyle;
                        markRow.CreateCell(0).SetCellValue("StepName");
                        markRow.CreateCell(1).SetCellValue("ActionName");
                        markRow.CreateCell(2).SetCellValue("ActionValue");
                        markRow.CreateCell(3).SetCellValue("Index");
                        markRow.CreateCell(4).SetCellValue($"！！！注意事项！！！\r\n(1)表格黑色背景部分请勿修改或删除！\r\n(2)请删除带“(例)”的示例数据！\r\n(3)sheet命名规则：商品名 属性值1 属性值2，如“牛肉面 一两 清汤”！\r\n(4)“工序(模型名称)”、“工序参数(模型配置)”、“工序参数值(输入值)”请查看设备工艺模版“{recodeName}”！");
                        markRow.GetCell(4).CellStyle = titleStyle;
                        IRow attrTitleRow = actionSheet.CreateRow(1);
                        attrTitleRow.RowStyle = titleStyle;
                        attrTitleRow.CreateCell(0).SetCellValue("属性");
                        attrTitleRow.CreateCell(1).SetCellValue("属性值");
                        var i = 2;
                        foreach (var attr in attrs)
                        {
                            var attrDataRow = actionSheet.CreateRow(i);
                            attrDataRow.RowStyle = dataStyle;
                            attrDataRow.CreateCell(0).SetCellValue(attr.Attr);
                            attrDataRow.CreateCell(1).SetCellValue(attr.AttrValue);
                            i++;
                        }
                        i += 2;
                        IRow actionTitleRow = actionSheet.CreateRow(i);
                        actionTitleRow.RowStyle = titleStyle;
                        actionTitleRow.CreateCell(0).SetCellValue("工序");
                        actionTitleRow.CreateCell(1).SetCellValue("工序参数");
                        actionTitleRow.CreateCell(2).SetCellValue("工序参数值");
                        actionTitleRow.CreateCell(3).SetCellValue("工序参数值排序");
                        var j = i + 1;
                        foreach (var item in action)
                        {
                            var actionJsonList = JsonConvert.DeserializeObject<List<ActionJsonModel>>(item.ActionJson);
                            foreach (var actionJson in actionJsonList)
                            {
                                IRow actionDataRow = actionSheet.CreateRow(j);
                                actionDataRow.RowStyle = dataStyle;
                                actionDataRow.CreateCell(0).SetCellValue(item.StepName);
                                actionDataRow.CreateCell(1).SetCellValue(actionJson.actionName);
                                actionDataRow.CreateCell(2).SetCellValue(actionJson.actionValue);
                                actionDataRow.CreateCell(3).SetCellValue(actionJson.index);
                                j++;
                            }
                        }
                        UpdateGoodSheet(goodSheet);
                        UpdateActionSheet(actionSheet);
                    }
                    var filePath = Path.Combine(tempFolder, $"{goodInfo.Name}_{deviceInfo.DeviceName}_{stopInfo.data.Name}.xlsx");
                    using var stream = new FileStream(filePath, FileMode.Create, FileAccess.Write);
                    workbook.Write(stream);
                }
            }
            ZipFile.CreateFromDirectory(tempFolder, zipFolder);
            var fileBase64 = Convert.ToBase64String(File.ReadAllBytes(zipFolder));
            return fileBase64;
        }

        /// <summary>
        /// 更新标题栏格式
        /// </summary>
        /// <param name="titleStyle"></param>
        /// <param name="font"></param>
        /// <returns></returns>
        public static void UpdateTitleStyle(ICellStyle titleStyle, IFont font)
        {
            titleStyle.FillForegroundColor = IndexedColors.Grey25Percent.Index;
            titleStyle.FillPattern = FillPattern.SolidForeground;
            titleStyle.VerticalAlignment = VerticalAlignment.Center;
            titleStyle.BorderBottom = BorderStyle.Thin;
            titleStyle.BottomBorderColor = IndexedColors.Black.Index;
            titleStyle.BorderLeft = BorderStyle.Thin;
            titleStyle.LeftBorderColor = IndexedColors.Black.Index;
            titleStyle.BorderRight = BorderStyle.Thin;
            titleStyle.RightBorderColor = IndexedColors.Black.Index;
            titleStyle.BorderTop = BorderStyle.Thin;
            titleStyle.TopBorderColor = IndexedColors.Black.Index;
            font.Boldweight = (short)FontBoldWeight.Bold;
            titleStyle.SetFont(font);
        }
        /// <summary>
        /// 更新数据栏格式
        /// </summary>
        /// <param name="dataStyle"></param>
        /// <returns></returns>
        public static void UpdateDataStyle(ICellStyle dataStyle)
        {
            dataStyle.FillPattern = FillPattern.SolidForeground;
            dataStyle.VerticalAlignment = VerticalAlignment.Center;
            dataStyle.BorderBottom = BorderStyle.Thin;
            dataStyle.BottomBorderColor = IndexedColors.Black.Index;
            dataStyle.BorderLeft = BorderStyle.Thin;
            dataStyle.LeftBorderColor = IndexedColors.Black.Index;
            dataStyle.BorderRight = BorderStyle.Thin;
            dataStyle.RightBorderColor = IndexedColors.Black.Index;
            dataStyle.BorderTop = BorderStyle.Thin;
            dataStyle.TopBorderColor = IndexedColors.Black.Index;
        }
        /// <summary>
        /// 更新商品sheet行高，列宽
        /// </summary>
        /// <param name="goodSheet"></param>
        public static void UpdateGoodSheet(ISheet goodSheet)
        {
            goodSheet.SetColumnWidth(0, 15 * 256);
            goodSheet.SetColumnWidth(1, 15 * 256);
            goodSheet.SetColumnWidth(2, 15 * 256);
            goodSheet.SetColumnWidth(3, 15 * 256);
            goodSheet.SetColumnWidth(4, 15 * 256);
            goodSheet.SetColumnWidth(5, 15 * 256);
            goodSheet.SetColumnWidth(6, 30 * 256);
            goodSheet.SetColumnWidth(7, 150 * 256);
            goodSheet.DefaultRowHeight = 30 * 20;
            goodSheet.GetRow(0).Height = 30 * 20;
            goodSheet.GetRow(1).Height = 80 * 20;
        }
        /// <summary>
        /// 更新工艺sheet行高，列宽
        /// </summary>
        /// <param name="actionSheet"></param>
        public static void UpdateActionSheet(ISheet actionSheet)
        {
            actionSheet.SetColumnWidth(0, 15 * 256);
            actionSheet.SetColumnWidth(1, 15 * 256);
            actionSheet.SetColumnWidth(2, 15 * 256);
            actionSheet.SetColumnWidth(3, 15 * 256);
            actionSheet.SetColumnWidth(4, 180 * 256);
            actionSheet.DefaultRowHeight = 30 * 20;
            actionSheet.AddMergedRegion(new CellRangeAddress(0, 1, 4, 4));
        }
        /// <summary>
        /// excel插入图片
        /// </summary>
        /// <param name="sheet"></param>
        /// <param name="imagePath"></param>
        /// <param name="rowIndex"></param>
        /// <param name="columnIndex"></param>
        /// <returns></returns>
        public static async Task InsertImage(ISheet sheet, string imagePath, int rowIndex, int columnIndex)
        {
            using HttpClient client = new();
            byte[] imageData = null;
            HttpResponseMessage response = await client.GetAsync(imagePath);
            if (response.IsSuccessStatusCode)
            {
                using HttpContent content = response.Content;
                imageData = await content.ReadAsByteArrayAsync();
            }
            else
            {
                imageData = null;
            }
            if (imageData != null)
            {
                // 添加图片到工作表
                int pictureIdx = sheet.Workbook.AddPicture(imageData, PictureType.JPEG);
                // 创建图片位置
                IDrawing patriarch = sheet.CreateDrawingPatriarch();
                XSSFClientAnchor anchor = new(0, 0, 0, 0, columnIndex, rowIndex, columnIndex + 1, rowIndex + 1);
                anchor.AnchorType = AnchorType.MoveAndResize;
                IPicture picture = patriarch.CreatePicture(anchor, pictureIdx);
                // 自动调整图片大小
                //picture.Resize();
            }
        }

        public static async Task DownloadRemoteFile(string url, string tempFolder, string fileName)
        {
            using (var httpClient = new HttpClient())
            {
                using var response = await httpClient.GetAsync(url, HttpCompletionOption.ResponseHeadersRead);
                response.EnsureSuccessStatusCode();
                using var stream = await response.Content.ReadAsStreamAsync();
                var tempFile = Path.Combine(tempFolder, $"{fileName}");
                using var fileStream = new FileStream(tempFile, FileMode.Create, FileAccess.Write, FileShare.None);
                await stream.CopyToAsync(fileStream);
            }
        }

        /// <summary>
        /// 商品工艺导出
        /// </summary>
        /// <param name="exportDto"></param>
        /// <returns></returns>
        public async Task<string> GoodsSimpleExport(GoodsTechnologyExportDto exportDto)
        {
            var goodsInfo = new List<BPA_GoodsInfo>();
            goodsInfo = exportDto.GoodsId.Count > 0
                ? await _db.Queryable<BPA_GoodsInfo>().Where(t => exportDto.GoodsId.Contains(t.Id)).ToListAsync()
                : await _db.Queryable<BPA_GoodsInfo>().Where(t => exportDto.Ids.Contains(t.Id)).ToListAsync();
            var goodsAttributeList = await _db.Queryable<BPA_GoodsAttribute>().ToListAsync();
            var goodsAttributeValueList = await _db.Queryable<BPA_GoodsAttributeValue>().Where(t => goodsAttributeList.Select(a => a.Id).Contains(t.GoodsAttributeId)).OrderBy(a => a.Sort).ToListAsync();
            var goodsTechnologyActionList = await _db.Queryable<BPA_GoodsTechnologyAction>().Where(t => t.DeviceId == exportDto.DeviceId).ToListAsync();
            IWorkbook workbook = new XSSFWorkbook();
            ICellStyle style = workbook.CreateCellStyle();
            style.FillForegroundColor = IndexedColors.Grey25Percent.Index;
            style.FillPattern = FillPattern.SolidForeground;
            style.VerticalAlignment = VerticalAlignment.Center;
            style.BorderBottom = BorderStyle.Thin;
            style.BottomBorderColor = IndexedColors.Black.Index;
            style.BorderLeft = BorderStyle.Thin;
            style.LeftBorderColor = IndexedColors.Black.Index;
            style.BorderRight = BorderStyle.Thin;
            style.RightBorderColor = IndexedColors.Black.Index;
            style.BorderTop = BorderStyle.Thin;
            style.TopBorderColor = IndexedColors.Black.Index;
            string[] charsToReplace = new string[] { "\\", "/", "?", "*", "[", "]" };
            foreach (var good in goodsInfo)
            {
                foreach (string c in charsToReplace)
                {
                    good.Name = good.Name.Replace(c, " ");
                }
                ISheet sheet = workbook.CreateSheet(good.Name);
                IRow headerRow = sheet.CreateRow(0);
                headerRow.CreateCell(0).SetCellValue("商品名称");
                headerRow.CreateCell(1).SetCellValue("属性组合");
                headerRow.CreateCell(2).SetCellValue("工序");
                headerRow.CreateCell(3).SetCellValue("工序参数");
                headerRow.CreateCell(4).SetCellValue("参数值");
                headerRow.GetCell(0).CellStyle = style;
                headerRow.GetCell(1).CellStyle = style;
                headerRow.GetCell(2).CellStyle = style;
                headerRow.GetCell(3).CellStyle = style;
                headerRow.GetCell(4).CellStyle = style;
                var actionList = goodsTechnologyActionList.Where(t => t.GoodsId == good.Id).OrderBy(a => a.GoodsAttributeId).OrderBy(a => a.Order).ToList();
                var k = 1;
                for (int i = 0; i < actionList.Count; i++)
                {
                    var stepName = actionList[i].StepName;
                    var attributeNames = "";
                    var attributeNameList = new List<string>();
                    var goodsAttributeIdList = actionList[i].GoodsAttributeId.Split(',');
                    foreach (var goodsAttributeId in goodsAttributeIdList)
                    {
                        attributeNameList.Add(goodsAttributeValueList.FirstOrDefault(t => t.Id == goodsAttributeId)?.AttributeValue);
                    }
                    attributeNames = string.Join("-", attributeNameList);
                    var actionJsonList = JsonConvert.DeserializeObject<List<ActionJsonModel>>(actionList[i].ActionJson);
                    for (int j = 0; j < actionJsonList.Count; j++)
                    {
                        IRow dataRow = sheet.CreateRow(k);
                        dataRow.CreateCell(0).SetCellValue(good.Name);
                        dataRow.CreateCell(1).SetCellValue(attributeNames);
                        dataRow.CreateCell(2).SetCellValue(stepName);
                        dataRow.CreateCell(3).SetCellValue(actionJsonList[j].actionName);
                        dataRow.CreateCell(4).SetCellValue(actionJsonList[j].actionValue);
                        k++;
                    }
                }
            }
            var tempFolder = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "GoodsTechnologyData");
            if (!Directory.Exists(tempFolder))
                Directory.CreateDirectory(tempFolder);
            var time = DateTime.Now.ToString("yyyy年MM月dd日HH时mm分ss秒");
            var filePath = Path.Combine(tempFolder, $"商品工艺-{time}.xlsx");
            using (var stream = new FileStream(filePath, FileMode.Create, FileAccess.Write))
            {
                workbook.Write(stream);
            }
            var fileBase64 = Convert.ToBase64String(File.ReadAllBytes(filePath));
            return fileBase64;
        }

        public static List<string> GenerateCombinations(Dictionary<string, List<string>> values, bool isId, BPA_GoodsInfo goodInfo, BPA_DeviceInfo deviceInfo)
        {
            var attributes = values.Keys.ToList();
            var results = new List<List<string>>();
            var currentCombo = new List<string>();
            Generate(values, attributes, results, currentCombo, 0);
            return (from item in results
                    let a = isId ? goodInfo.Id + ",(" + string.Join(",", item) + ")," + deviceInfo.Id : goodInfo.Name + "-(" + string.Join("-", item) + ")-" + deviceInfo.DeviceName
                    select a).ToList();
        }

        private static void Generate(Dictionary<string, List<string>> values, List<string> attributes, List<List<string>> results, List<string> currentCombo, int index)
        {
            if (index >= attributes.Count)
            {
                results.Add(new List<string>(currentCombo));
                return;
            }
            var attribute = attributes[index];
            foreach (var value in values[attribute])
            {
                currentCombo.Add(value);
                Generate(values, attributes, results, currentCombo, index + 1);
                currentCombo.RemoveAt(currentCombo.Count - 1);
            }
        }

        public string GetValueStr(string str)
        {
            try
            {
                var obj = JsonConvert.DeserializeObject<List<ActionValueJsonDto>>(str);
                var result = String.Join(",", obj.Select(x => x.ActionValueName).ToArray());
                return result;
            }
            catch (Exception e)
            {
                return "";
            }
        }
        public string CharacterConversion(string str)
        {
            switch (str)
            {
                case "下拉框":
                    return "select";
                case "数字输入框":
                    return "digit";
                case "文本框":
                    return "text";
                case "开关":
                    return "switch";
                case "文字域":
                    return "textarea";
                default:
                    break;
            }

            return "";
        }
        /// <summary>
        /// 设置excel的下拉框
        /// </summary>
        /// <param name="sheet"></param>
        /// <param name="col"></param>
        /// <param name="selectData"></param>
        private void SetExcelSelect(ISheet sheet, List<string> selectData, int firstRow, int lastRow, int firstCol, int lastCol)
        {
            var count = selectData.Sum(item => item.Count());


            if (count > 180)
            {
                return;
            }
            //设置生成下拉框的行和列
            var cellRegions = new CellRangeAddressList(1, 65535, firstCol, lastCol);
            //设置 下拉框内容
            DVConstraint constraint = DVConstraint.CreateExplicitListConstraint(selectData.ToArray());
            //绑定下拉框和作用区域，并设置错误提示信息
            HSSFDataValidation dataValidate = new HSSFDataValidation(cellRegions, constraint);
            dataValidate.CreateErrorBox("输入不合法", "请输入或选择下拉列表中的值。");
            dataValidate.ShowPromptBox = true;
            sheet.AddValidationData(dataValidate);
        }

        /// <summary>   
        /// 将 Stream 写入文件   
        /// </summary>   
        private void StreamToFile(byte[] bytes, string filepath)
        {
            // 把 byte[] 写入文件   
            FileStream fs = new FileStream(filepath, FileMode.Create);
            BinaryWriter bw = new BinaryWriter(fs);
            bw.Write(bytes);
            bw.Close();
            fs.Close();
        }
        /// <summary>
        /// 获取 shtte name
        /// </summary>
        /// <param name="stream"></param>
        /// <param name="fileType"></param>
        /// <param name="strMsg"></param>
        /// <param name="sheetName"></param>
        /// <returns></returns>
        private List<string> ExcelToShtteName(Stream stream, string fileType, bool isGY = true)
        {

            var result = new List<string>();
            ISheet sheet = null;
            IWorkbook workbook = null;
            var shellCount = 0;
            try
            {
                #region 判断excel版本
                //2007以上版本excel
                if (fileType == ".xlsx")
                {
                    workbook = new XSSFWorkbook(stream);
                    shellCount = ((XSSFWorkbook)workbook).Count;
                }
                //2007以下版本excel
                else if (fileType == ".xls")
                {
                    workbook = new HSSFWorkbook(stream);
                    shellCount = ((HSSFWorkbook)workbook).Count;
                }
                else
                {
                    throw Oops.Oh($"传入的不是Excel文件");
                }
                #endregion

                for (int sheetI = 0; sheetI < shellCount; sheetI++)
                {
                    sheet = workbook.GetSheetAt(sheetI);

                    if (isGY)
                    {
                        if (sheet.SheetName.Contains("工艺"))
                        {
                            result.Add(sheet.SheetName);
                        }
                    }
                    else
                    {
                        result.Add(sheet.SheetName);
                    }

                }
            }
            catch (Exception ex)
            {
                throw Oops.Oh($"Excel操作错误" + ex.Message);
            }
            return result;
        }
        private List<string> GetSheetNameList(Stream stream)
        {
            var result = new List<string>();
            try
            {
                IWorkbook workbook = new XSSFWorkbook(stream);
                for (int i = 0; i < ((XSSFWorkbook)workbook).Count; i++)
                {
                    result.Add(workbook.GetSheetAt(i).SheetName);
                }
            }
            catch (Exception ex)
            {
                throw Oops.Oh($"Excel读取错误" + ex.Message);
            }
            return result;
        }
        /// <summary>
        /// 将Excel单表转为Datatable
        /// </summary>
        /// <param name="stream"></param>
        /// <param name="fileType"></param>
        /// <param name="strMsg"></param>
        /// <param name="sheetName"></param>
        /// <returns></returns>
        public async Task<List<ExcelDtoImagDto>> ExcelToDatatable(Stream stream, string fileType, string sheetName = null)
        {
            var result = new List<ExcelDtoImagDto>();
            ISheet sheet = null;
            IWorkbook workbook = null;

            try
            {
                #region 判断excel版本
                //2007以上版本excel
                if (fileType == ".xlsx")
                {
                    workbook = new XSSFWorkbook(stream);
                }
                //2007以下版本excel
                else if (fileType == ".xls")
                {
                    workbook = new HSSFWorkbook(stream);
                }
                else
                {
                    throw new Exception("传入的不是Excel文件！");
                }
                #endregion
                Console.WriteLine("已确定解析格式，开始解析");
                if (workbook == null)
                {
                    Console.WriteLine("已确定解析格式，工具");
                }

                IList pictures = workbook.GetAllPictures();
                Console.WriteLine("已确定解析格式，成功解析" + JsonConvert.SerializeObject(pictures));
                int i = 1;
                foreach (XSSFPictureData pic in pictures)
                {
                    Console.WriteLine("讲数据流转化为图片");
                    Image jpg1 = Image.FromStream(new MemoryStream(pic.Data));//从pic.Data数据流创建图片
                    Console.WriteLine("转化成功");
                    var imagePath = await GetCosImgesURL(PhotoImageInsert(jpg1));
                    Console.WriteLine("图片路径" + imagePath);
                    result.Add(new ExcelDtoImagDto()
                    {
                        ImgesPath = imagePath,
                        Index = i
                    });
                    i++;
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }
            return result;
        }

        //将Image转换成流数据，并保存为byte[] 
        public byte[] PhotoImageInsert(Image imgPhoto)
        {
            MemoryStream mstream = new MemoryStream();
            imgPhoto.Save(mstream, ImageFormat.Bmp);
            byte[] byData = new Byte[mstream.Length];
            mstream.Position = 0;
            mstream.Read(byData, 0, byData.Length); mstream.Close();
            return byData;
        }
        public async Task<string> GetCosImgesURL(byte[] data)
        {
            var result = "";
            TransferUploadObjectModel m = new TransferUploadObjectModel(_db);
            /// 高级接口上传对象
            result = await m.TransferUploadFile(new FileInputInfoDto()
            {
                Directory = "goods",
                FileExtension = "png",
                Method = "PUT",
            }, data);
            return result;
        }

        private string GetNumber2(int Length = 10)
        {
            byte[] buffer = Guid.NewGuid().ToByteArray();
            var ram = BitConverter.ToInt64(buffer, 0);
            var str = string.Format("{0}", ram.ToString().Substring(0, Length));
            return str;
        }

        public async Task<bool> UpdateBatching()
        {
            try
            {
                _db.Ado.BeginTran();
                var actionList = await _db.Queryable<BPA_GoodsTechnologyAction>().Where(t => t.ChnologyId == "117c2692-247c-414a-a6ff-c42f2f8c32fb").ToListAsync();
                foreach (var action in actionList)
                {
                    var json = JsonConvert.DeserializeObject<List<NewActionJsonModel>>(action.ActionJson);
                    var newlist = new List<ActionJsonModel>();
                    var model1 = new ActionJsonModel
                    {
                        actionName = "物料名称",
                        actionValue = json[0].actionName,
                        technologyactionId = "c22314a3-aef0-49f6-8e80-4c55a2b1d13a",
                        index = "0"
                    };
                    newlist.Add(model1);
                    var model2 = new ActionJsonModel
                    {
                        actionName = "物料重量",
                        actionValue = json[0].actionValue,
                        technologyactionId = "2d7fcf6e-d18e-497a-a942-7169cf75c98c",
                        index = "1"
                    };
                    newlist.Add(model2);
                    action.ActionJson = JsonConvert.SerializeObject(newlist);
                }
                await _db.Updateable(actionList).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception)
            {
                _db.Ado.RollbackTran();
                throw;
            }
        }

        public async Task<bool> UpdateAction()
        {
            try
            {
                _db.Ado.BeginTran();
                var actionList = await _db.Queryable<BPA_GoodsTechnologyAction>().Where(t => t.GroupId == "49577cdf-9ce2-4329-8b06-6f6b1fe7e3a5" && t.StepName == "主料").ToListAsync();
                foreach (var action in actionList)
                {
                    var jsonList = JsonConvert.DeserializeObject<List<NewActionJsonModel>>(action.ActionJson);
                    var newlist = new List<ActionJsonModel>();
                    var index = 0;
                    foreach (var json in jsonList)
                    {
                        if(json.actionName == "炒制位置")
                        {
                            newlist.Add(new ActionJsonModel
                            {
                                actionName = json.actionName,
                                actionValue = json.actionValue,
                                technologyactionId = "732763c8-065f-4819-a276-a962fc1b0543",
                                index = index.ToString()
                            });
                        }
                        else
                        {
                            newlist.Add(new ActionJsonModel
                            {
                                actionName = json.actionName,
                                actionValue = json.actionValue,
                                technologyactionId = json.technologyactionId,
                                index = index.ToString()
                            });
                        }
                        index++;
                    }
                    if (!newlist.Select(t => t.actionName).Contains("炒制位置"))
                    {
                        newlist.Add(new ActionJsonModel
                        {
                            actionName = "炒制位置",
                            actionValue = "炒菜位1",
                            technologyactionId = "732763c8-065f-4819-a276-a962fc1b0543",
                            index = "5"
                        });
                    }
                    action.ActionJson = JsonConvert.SerializeObject(newlist);
                }
                await _db.Updateable(actionList).ExecuteCommandAsync();
                _db.Ado.CommitTran();
                return true;
            }
            catch (Exception)
            {
                _db.Ado.RollbackTran();
                throw;
            }
        }
    }
}
