﻿using BPA.SAAS.Manage.Application.DataBase.Dtos.Recipe;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Core.DataBase;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase.Services
{
    public class RecipeService : IRecipeService, ITransient
    {
        private readonly ISqlSugarClient _db;
        public RecipeService(ISqlSugarClient db)
        {
            _db = db;
        }

        /// <summary>
        /// 获取配方分组
        /// </summary>
        /// <returns></returns>
        public async Task<List<RecipeGroupDto>> GetRecipeGroupList()
        {
            return await _db.Queryable<BPA_RecipeGroup>().OrderBy(t => t.Sort).Select(t => new RecipeGroupDto
            {
                Id = t.Id,
                Name = t.Name,
                Sort = t.Sort
            }).ToListAsync();
        }

        /// <summary>
        /// 更新配方分组
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdataRecipeGroup(RecipeGroupDto dto)
        {
            try
            {
                var res = 0;
                if (dto.Id == null)
                {
                    var group = await _db.Queryable<BPA_RecipeGroup>().FirstAsync(t => t.Name == dto.Name);
                    if (group != null)
                        throw Oops.Oh($"{dto.Name}已存在");
                    group = new BPA_RecipeGroup
                    {
                        Name = dto.Name,
                        Sort = dto.Sort
                    };
                    res = await _db.Insertable(group).CallEntityMethod(t => t.Create()).ExecuteCommandAsync();
                }
                else
                {
                    var group = await _db.Queryable<BPA_RecipeGroup>().FirstAsync(t => t.Id == dto.Id);
                    if (group == null)
                        throw Oops.Oh($"{dto.Name}不存在或已删除");
                    group.Name = dto.Name;
                    group.Sort = dto.Sort;
                    res = await _db.Updateable(group).ExecuteCommandAsync();
                }
                return res > 0;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 删除配方分组
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<bool> DeleteRecipeGroup(string id)
        {
            try
            {
                var group = await _db.Queryable<BPA_RecipeGroup>().FirstAsync(t => t.Id == id);
                var res = await _db.Deleteable(group).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 通过配方分组获取配方信息
        /// </summary>
        /// <param name="recipeGroupId"></param>
        /// <returns></returns>
        public async Task<List<RecipeInfoDto>> GetRecipeInfoList(string recipeGroupId)
        {
            return await _db.Queryable<BPA_RecipeInfo>().Where(t => t.RecipeGroupId == recipeGroupId).OrderBy(a => a.Sort).Select(t => new RecipeInfoDto
            {
                Id = t.Id,
                RecipeGroupId = t.RecipeGroupId,
                Name = t.Name,
                RecipeType = t.RecipeType,
                BenchmarkGram = t.BenchmarkGram,
                Sort = t.Sort
            }).ToListAsync();
        }
    }
}
